from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # sequence (positiveInt)
    from spark_auto_mapper_fhir.fhir_types.positive_int import FhirPositiveInt

    # type_ (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for type_
    # Import for CodeableConcept for type_
    from spark_auto_mapper_fhir.value_sets.example_diagnosis_type_codes import (
        ExampleDiagnosisTypeCodesCode,
    )

    # End Import for CodeableConcept for type_
    # onAdmission (CodeableConcept)
    # End Import for References for onAdmission
    # Import for CodeableConcept for onAdmission
    from spark_auto_mapper_fhir.value_sets.example_diagnosis_on_admission_codes import (
        ExampleDiagnosisOnAdmissionCodesCode,
    )

    # End Import for CodeableConcept for onAdmission
    # packageCode (CodeableConcept)
    # End Import for References for packageCode
    # Import for CodeableConcept for packageCode
    from spark_auto_mapper_fhir.value_sets.example_diagnosis_related_group_codes import (
        ExampleDiagnosisRelatedGroupCodesCode,
    )

    # End Import for CodeableConcept for packageCode
    # diagnosisCodeableConcept (CodeableConcept)
    # End Import for References for diagnosisCodeableConcept
    # Import for CodeableConcept for diagnosisCodeableConcept
    from spark_auto_mapper_fhir.value_sets.generic_type import GenericTypeCode

    # End Import for CodeableConcept for diagnosisCodeableConcept
    # diagnosisReference (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for diagnosisReference
    from spark_auto_mapper_fhir.resources.condition import Condition


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class ClaimDiagnosis(FhirBackboneElementBase):
    """
    Claim.Diagnosis
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        sequence: FhirPositiveInt,
        type_: Optional[
            FhirList[CodeableConcept[ExampleDiagnosisTypeCodesCode]]
        ] = None,
        onAdmission: Optional[
            CodeableConcept[ExampleDiagnosisOnAdmissionCodesCode]
        ] = None,
        packageCode: Optional[
            CodeableConcept[ExampleDiagnosisRelatedGroupCodesCode]
        ] = None,
        diagnosisCodeableConcept: Optional[CodeableConcept[GenericTypeCode]] = None,
        diagnosisReference: Optional[Reference[Union[Condition]]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param sequence: A number to uniquely identify diagnosis entries.
            :param type_: When the condition was observed or the relative ranking.
            :param onAdmission: Indication of whether the diagnosis was present on admission to a facility.
            :param packageCode: A package billing code or bundle code used to group products and services to a
        particular health condition (such as heart attack) which is based on a
        predetermined grouping code system.
            :param diagnosisCodeableConcept: None
            :param diagnosisReference: None
        """
        super().__init__(
            id_=id_,
            extension=extension,
            sequence=sequence,
            type_=type_,
            onAdmission=onAdmission,
            packageCode=packageCode,
            diagnosisCodeableConcept=diagnosisCodeableConcept,
            diagnosisReference=diagnosisReference,
        )
