from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # itemSequence (positiveInt)
    from spark_auto_mapper_fhir.fhir_types.positive_int import FhirPositiveInt

    # detailSequence (positiveInt)
    # subDetailSequence (positiveInt)
    # code (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for code
    # Import for CodeableConcept for code
    from spark_auto_mapper_fhir.value_sets.adjudication_error_codes import (
        AdjudicationErrorCodesCode,
    )

    # End Import for CodeableConcept for code


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class ClaimResponseError(FhirBackboneElementBase):
    """
    ClaimResponse.Error
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        itemSequence: Optional[FhirPositiveInt] = None,
        detailSequence: Optional[FhirPositiveInt] = None,
        subDetailSequence: Optional[FhirPositiveInt] = None,
        code: CodeableConcept[AdjudicationErrorCodesCode],
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param itemSequence: The sequence number of the line item submitted which contains the error. This
        value is omitted when the error occurs outside of the item structure.
            :param detailSequence: The sequence number of the detail within the line item submitted which
        contains the error. This value is omitted when the error occurs outside of the
        item structure.
            :param subDetailSequence: The sequence number of the sub-detail within the detail within the line item
        submitted which contains the error. This value is omitted when the error
        occurs outside of the item structure.
            :param code: An error code, from a specified code system, which details why the claim could
        not be adjudicated.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            itemSequence=itemSequence,
            detailSequence=detailSequence,
            subDetailSequence=subDetailSequence,
            code=code,
        )
