from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # code (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for code
    # Import for CodeableConcept for code
    from spark_auto_mapper_fhir.value_sets.investigation_type import (
        InvestigationTypeCode,
    )

    # End Import for CodeableConcept for code
    # item (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for item
    from spark_auto_mapper_fhir.resources.observation import Observation
    from spark_auto_mapper_fhir.resources.questionnaire_response import (
        QuestionnaireResponse,
    )
    from spark_auto_mapper_fhir.resources.family_member_history import (
        FamilyMemberHistory,
    )
    from spark_auto_mapper_fhir.resources.diagnostic_report import DiagnosticReport
    from spark_auto_mapper_fhir.resources.risk_assessment import RiskAssessment
    from spark_auto_mapper_fhir.resources.imaging_study import ImagingStudy
    from spark_auto_mapper_fhir.resources.media import Media


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class ClinicalImpressionInvestigation(FhirBackboneElementBase):
    """
    ClinicalImpression.Investigation
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        code: CodeableConcept[InvestigationTypeCode],
        item: Optional[
            FhirList[
                Reference[
                    Union[
                        Observation,
                        QuestionnaireResponse,
                        FamilyMemberHistory,
                        DiagnosticReport,
                        RiskAssessment,
                        ImagingStudy,
                        Media,
                    ]
                ]
            ]
        ] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param code: A name/code for the group ("set") of investigations. Typically, this will be
        something like "signs", "symptoms", "clinical", "diagnostic", but the list is
        not constrained, and others such groups such as
        (exposure|family|travel|nutritional) history may be used.
            :param item: A record of a specific investigation that was undertaken.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            code=code,
            item=item,
        )
