from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId
from spark_auto_mapper_fhir.fhir_types.uri import FhirUri

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # code (code)
    from spark_auto_mapper_fhir.complex_types.code import code

    # uri (uri)
    # description (string)
    # type_ (PropertyType)
    from spark_auto_mapper_fhir.value_sets.property_type import PropertyTypeCode


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class CodeSystemProperty(FhirBackboneElementBase):
    """
    CodeSystem.Property
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        code: code,
        uri: Optional[FhirUri] = None,
        description: Optional[FhirString] = None,
        type_: PropertyTypeCode,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param code: A code that is used to identify the property. The code is used internally (in
        CodeSystem.concept.property.code) and also externally, such as in property
        filters.
            :param uri: Reference to the formal meaning of the property. One possible source of
        meaning is the [Concept Properties](codesystem-concept-properties.html) code
        system.
            :param description: A description of the property- why it is defined, and how its value might be
        used.
            :param type_: The type of the property value. Properties of type "code" contain a code
        defined by the code system (e.g. a reference to another defined concept).
        """
        super().__init__(
            id_=id_,
            extension=extension,
            code=code,
            uri=uri,
            description=description,
            type_=type_,
        )
