from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # mode (CompositionAttestationMode)
    from spark_auto_mapper_fhir.value_sets.composition_attestation_mode import (
        CompositionAttestationModeCode,
    )

    # time (dateTime)
    # party (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for party
    from spark_auto_mapper_fhir.resources.patient import Patient
    from spark_auto_mapper_fhir.resources.related_person import RelatedPerson
    from spark_auto_mapper_fhir.resources.practitioner import Practitioner
    from spark_auto_mapper_fhir.resources.practitioner_role import PractitionerRole
    from spark_auto_mapper_fhir.resources.organization import Organization


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class CompositionAttester(FhirBackboneElementBase):
    """
    Composition.Attester
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        mode: CompositionAttestationModeCode,
        time: Optional[FhirDateTime] = None,
        party: Optional[
            Reference[
                Union[
                    Patient, RelatedPerson, Practitioner, PractitionerRole, Organization
                ]
            ]
        ] = None,
    ) -> None:
        """

        :param id_: id of resource
        :param extension: extensions
        :param mode: The type of attestation the authenticator offers.
        :param time: When the composition was attested by the party.
        :param party: Who attested the composition in the specified way.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            mode=mode,
            time=time,
            party=party,
        )
