from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # summary (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for summary
    # Import for CodeableConcept for summary
    from spark_auto_mapper_fhir.value_sets.condition_stage import ConditionStageCode

    # End Import for CodeableConcept for summary
    # assessment (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for assessment
    from spark_auto_mapper_fhir.resources.clinical_impression import ClinicalImpression
    from spark_auto_mapper_fhir.resources.diagnostic_report import DiagnosticReport
    from spark_auto_mapper_fhir.resources.observation import Observation

    # type_ (CodeableConcept)
    # End Import for References for type_
    # Import for CodeableConcept for type_
    from spark_auto_mapper_fhir.value_sets.condition_stage_type import (
        ConditionStageTypeCode,
    )

    # End Import for CodeableConcept for type_


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class ConditionStage(FhirBackboneElementBase):
    """
    Condition.Stage
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        summary: Optional[CodeableConcept[ConditionStageCode]] = None,
        assessment: Optional[
            FhirList[
                Reference[Union[ClinicalImpression, DiagnosticReport, Observation]]
            ]
        ] = None,
        type_: Optional[CodeableConcept[ConditionStageTypeCode]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param summary: A simple summary of the stage such as "Stage 3". The determination of the
        stage is disease-specific.
            :param assessment: Reference to a formal record of the evidence on which the staging assessment
        is based.
            :param type_: The kind of staging, such as pathological or clinical staging.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            summary=summary,
            assessment=assessment,
            type_=type_,
        )
