from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId
from spark_auto_mapper_fhir.resources.resource import Resource

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # reference (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for reference
    # code (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for code
    # Import for CodeableConcept for code
    from spark_auto_mapper_fhir.value_sets.contract_resource_asset_context_codes import (
        ContractResourceAssetContextCodesCode,
    )

    # End Import for CodeableConcept for code
    # text (string)


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class ContractContext(FhirBackboneElementBase):
    """
    Contract.Context
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        reference: Optional[Reference[Union[Resource]]] = None,
        code: Optional[
            FhirList[CodeableConcept[ContractResourceAssetContextCodesCode]]
        ] = None,
        text: Optional[FhirString] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param reference: Asset context reference may include the creator, custodian, or owning Person
        or Organization (e.g., bank, repository),  location held, e.g., building,
        jurisdiction.
            :param code: Coded representation of the context generally or of the Referenced entity,
        such as the asset holder type or location.
            :param text: Context description.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            reference=reference,
            code=code,
            text=text,
        )
