from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # condition (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for condition
    from spark_auto_mapper_fhir.resources.condition import Condition
    from spark_auto_mapper_fhir.resources.procedure import Procedure

    # use (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for use
    # Import for CodeableConcept for use
    from spark_auto_mapper_fhir.value_sets.diagnosis_role import DiagnosisRoleCode

    # End Import for CodeableConcept for use
    # rank (positiveInt)
    from spark_auto_mapper_fhir.fhir_types.positive_int import FhirPositiveInt


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class EncounterDiagnosis(FhirBackboneElementBase):
    """
    Encounter.Diagnosis
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        condition: Reference[Union[Condition, Procedure]],
        use: Optional[CodeableConcept[DiagnosisRoleCode]] = None,
        rank: Optional[FhirPositiveInt] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param condition: Reason the encounter takes place, as specified using information from another
        resource. For admissions, this is the admission diagnosis. The indication will
        typically be a Condition (with other resources referenced in the
        evidence.detail), or a Procedure.
            :param use: Role that this diagnosis has within the encounter (e.g. admission, billing,
        discharge …).
            :param rank: Ranking of the diagnosis (for each role type).
        """
        super().__init__(
            id_=id_,
            extension=extension,
            condition=condition,
            use=use,
            rank=rank,
        )
