from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.boolean import FhirBoolean
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # code (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for code
    # Import for CodeableConcept for code
    from spark_auto_mapper_fhir.value_sets.condition_or__problem_or__diagnosis_codes import (
        Condition_or_Problem_or_DiagnosisCodesCode,
    )

    # End Import for CodeableConcept for code
    # outcome (CodeableConcept)
    # End Import for References for outcome
    # Import for CodeableConcept for outcome
    from spark_auto_mapper_fhir.value_sets.condition_outcome_codes import (
        ConditionOutcomeCodesCode,
    )

    # End Import for CodeableConcept for outcome
    # contributedToDeath (boolean)
    # note (Annotation)
    from spark_auto_mapper_fhir.complex_types.annotation import Annotation

    # onsetAge (Age)
    from spark_auto_mapper_fhir.complex_types.age import Age

    # onsetRange (Range)
    from spark_auto_mapper_fhir.complex_types.range import Range

    # onsetPeriod (Period)
    from spark_auto_mapper_fhir.complex_types.period import Period

    # onsetString (string)


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class FamilyMemberHistoryCondition(FhirBackboneElementBase):
    """
    FamilyMemberHistory.Condition
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        code: CodeableConcept[Condition_or_Problem_or_DiagnosisCodesCode],
        outcome: Optional[CodeableConcept[ConditionOutcomeCodesCode]] = None,
        contributedToDeath: Optional[FhirBoolean] = None,
        note: Optional[FhirList[Annotation]] = None,
        onsetAge: Optional[Age] = None,
        onsetRange: Optional[Range] = None,
        onsetPeriod: Optional[Period] = None,
        onsetString: Optional[FhirString] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param code: The actual condition specified. Could be a coded condition (like MI or
        Diabetes) or a less specific string like 'cancer' depending on how much is
        known about the condition and the capabilities of the creating system.
            :param outcome: Indicates what happened following the condition.  If the condition resulted in
        death, deceased date is captured on the relation.
            :param contributedToDeath: This condition contributed to the cause of death of the related person. If
        contributedToDeath is not populated, then it is unknown.
            :param note: An area where general notes can be placed about this specific condition.
            :param onsetAge: None
            :param onsetRange: None
            :param onsetPeriod: None
            :param onsetString: None
        """
        super().__init__(
            id_=id_,
            extension=extension,
            code=code,
            outcome=outcome,
            contributedToDeath=contributedToDeath,
            note=note,
            onsetAge=onsetAge,
            onsetRange=onsetRange,
            onsetPeriod=onsetPeriod,
            onsetString=onsetString,
        )
