from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.boolean import FhirBoolean
from spark_auto_mapper_fhir.fhir_types.date import FhirDate
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.integer import FhirInteger
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # measure (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for measure
    # Import for CodeableConcept for measure
    from spark_auto_mapper_fhir.value_sets.loinc_codes import LOINCCodesCode

    # End Import for CodeableConcept for measure
    # detailQuantity (Quantity)
    from spark_auto_mapper_fhir.complex_types.quantity import Quantity

    # detailRange (Range)
    from spark_auto_mapper_fhir.complex_types.range import Range

    # detailCodeableConcept (CodeableConcept)
    # End Import for References for detailCodeableConcept
    # Import for CodeableConcept for detailCodeableConcept
    from spark_auto_mapper_fhir.value_sets.generic_type import GenericTypeCode

    # End Import for CodeableConcept for detailCodeableConcept
    # detailString (string)
    # detailBoolean (boolean)
    # detailInteger (integer)
    # detailRatio (Ratio)
    from spark_auto_mapper_fhir.complex_types.ratio import Ratio

    # dueDate (date)
    # dueDuration (Duration)
    from spark_auto_mapper_fhir.complex_types.duration import Duration


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class GoalTarget(FhirBackboneElementBase):
    """
    Goal.Target
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        measure: Optional[CodeableConcept[LOINCCodesCode]] = None,
        detailQuantity: Optional[Quantity] = None,
        detailRange: Optional[Range] = None,
        detailCodeableConcept: Optional[CodeableConcept[GenericTypeCode]] = None,
        detailString: Optional[FhirString] = None,
        detailBoolean: Optional[FhirBoolean] = None,
        detailInteger: Optional[FhirInteger] = None,
        detailRatio: Optional[Ratio] = None,
        dueDate: Optional[FhirDate] = None,
        dueDuration: Optional[Duration] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param measure: The parameter whose value is being tracked, e.g. body weight, blood pressure,
        or hemoglobin A1c level.
            :param detailQuantity: None
            :param detailRange: None
            :param detailCodeableConcept: None
            :param detailString: None
            :param detailBoolean: None
            :param detailInteger: None
            :param detailRatio: None
            :param dueDate: None
            :param dueDuration: None
        """
        super().__init__(
            id_=id_,
            extension=extension,
            measure=measure,
            detailQuantity=detailQuantity,
            detailRange=detailRange,
            detailCodeableConcept=detailCodeableConcept,
            detailString=detailString,
            detailBoolean=detailBoolean,
            detailInteger=detailInteger,
            detailRatio=detailRatio,
            dueDate=dueDate,
            dueDuration=dueDuration,
        )
