from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # uid (id)
    from spark_auto_mapper_fhir.complex_types.id import id

    # number (unsignedInt)
    from spark_auto_mapper_fhir.complex_types.unsigned_int import unsignedInt

    # modality (Coding)
    from spark_auto_mapper_fhir.complex_types.coding import Coding

    # End Import for References for modality
    # Import for CodeableConcept for modality
    from spark_auto_mapper_fhir.value_sets.generic_type import GenericTypeCode

    # End Import for CodeableConcept for modality
    # description (string)
    # numberOfInstances (unsignedInt)
    # endpoint (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for endpoint
    from spark_auto_mapper_fhir.resources.endpoint import Endpoint

    # bodySite (Coding)
    # End Import for References for bodySite
    # Import for CodeableConcept for bodySite
    from spark_auto_mapper_fhir.value_sets.snomedct_body_structures import (
        SNOMEDCTBodyStructuresCode,
    )

    # End Import for CodeableConcept for bodySite
    # laterality (Coding)
    # End Import for References for laterality
    # Import for CodeableConcept for laterality
    from spark_auto_mapper_fhir.value_sets.laterality import LateralityCode

    # End Import for CodeableConcept for laterality
    # specimen (Reference)
    # Imports for References for specimen
    from spark_auto_mapper_fhir.resources.specimen import Specimen

    # started (dateTime)
    # performer (ImagingStudy.Performer)
    from spark_auto_mapper_fhir.backbone_elements.imaging_study_performer import (
        ImagingStudyPerformer,
    )

    # instance (ImagingStudy.Instance)
    from spark_auto_mapper_fhir.backbone_elements.imaging_study_instance import (
        ImagingStudyInstance,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class ImagingStudySeries(FhirBackboneElementBase):
    """
    ImagingStudy.Series
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        uid: id,
        number: Optional[unsignedInt] = None,
        modality: Coding[GenericTypeCode],
        description: Optional[FhirString] = None,
        numberOfInstances: Optional[unsignedInt] = None,
        endpoint: Optional[FhirList[Reference[Union[Endpoint]]]] = None,
        bodySite: Optional[Coding[SNOMEDCTBodyStructuresCode]] = None,
        laterality: Optional[Coding[LateralityCode]] = None,
        specimen: Optional[FhirList[Reference[Union[Specimen]]]] = None,
        started: Optional[FhirDateTime] = None,
        performer: Optional[FhirList[ImagingStudyPerformer]] = None,
        instance: Optional[FhirList[ImagingStudyInstance]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param uid: The DICOM Series Instance UID for the series.
            :param number: The numeric identifier of this series in the study.
            :param modality: The modality of this series sequence.
            :param description: A description of the series.
            :param numberOfInstances: Number of SOP Instances in the Study. The value given may be larger than the
        number of instance elements this resource contains due to resource
        availability, security, or other factors. This element should be present if
        any instance elements are present.
            :param endpoint: The network service providing access (e.g., query, view, or retrieval) for
        this series. See implementation notes for information about using DICOM
        endpoints. A series-level endpoint, if present, has precedence over a study-
        level endpoint with the same Endpoint.connectionType.
            :param bodySite: The anatomic structures examined. See DICOM Part 16 Annex L (http://dicom.nema
        .org/medical/dicom/current/output/chtml/part16/chapter_L.html) for DICOM to
        SNOMED-CT mappings. The bodySite may indicate the laterality of body part
        imaged; if so, it shall be consistent with any content of
        ImagingStudy.series.laterality.
            :param laterality: The laterality of the (possibly paired) anatomic structures examined. E.g.,
        the left knee, both lungs, or unpaired abdomen. If present, shall be
        consistent with any laterality information indicated in
        ImagingStudy.series.bodySite.
            :param specimen: The specimen imaged, e.g., for whole slide imaging of a biopsy.
            :param started: The date and time the series was started.
            :param performer: Indicates who or what performed the series and how they were involved.
            :param instance: A single SOP instance within the series, e.g. an image, or presentation state.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            uid=uid,
            number=number,
            modality=modality,
            description=description,
            numberOfInstances=numberOfInstances,
            endpoint=endpoint,
            bodySite=bodySite,
            laterality=laterality,
            specimen=specimen,
            started=started,
            performer=performer,
            instance=instance,
        )
