from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId
from spark_auto_mapper_fhir.resources.resource import Resource

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # vaccineCode (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for vaccineCode
    # Import for CodeableConcept for vaccineCode
    from spark_auto_mapper_fhir.value_sets.vaccine_administered_value_set import (
        VaccineAdministeredValueSetCode,
    )

    # End Import for CodeableConcept for vaccineCode
    # targetDisease (CodeableConcept)
    # End Import for References for targetDisease
    # Import for CodeableConcept for targetDisease
    from spark_auto_mapper_fhir.value_sets.immunization_recommendation_target_disease_codes import (
        ImmunizationRecommendationTargetDiseaseCodesCode,
    )

    # End Import for CodeableConcept for targetDisease
    # contraindicatedVaccineCode (CodeableConcept)
    # End Import for References for contraindicatedVaccineCode
    # Import for CodeableConcept for contraindicatedVaccineCode
    # End Import for CodeableConcept for contraindicatedVaccineCode
    # forecastStatus (CodeableConcept)
    # End Import for References for forecastStatus
    # Import for CodeableConcept for forecastStatus
    from spark_auto_mapper_fhir.value_sets.immunization_recommendation_status_codes import (
        ImmunizationRecommendationStatusCodesCode,
    )

    # End Import for CodeableConcept for forecastStatus
    # forecastReason (CodeableConcept)
    # End Import for References for forecastReason
    # Import for CodeableConcept for forecastReason
    from spark_auto_mapper_fhir.value_sets.immunization_recommendation_reason_codes import (
        ImmunizationRecommendationReasonCodesCode,
    )

    # End Import for CodeableConcept for forecastReason
    # dateCriterion (ImmunizationRecommendation.DateCriterion)
    from spark_auto_mapper_fhir.backbone_elements.immunization_recommendation_date_criterion import (
        ImmunizationRecommendationDateCriterion,
    )

    # description (string)
    # series (string)
    # supportingImmunization (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for supportingImmunization
    from spark_auto_mapper_fhir.resources.immunization import Immunization
    from spark_auto_mapper_fhir.resources.immunization_evaluation import (
        ImmunizationEvaluation,
    )

    # supportingPatientInformation (Reference)
    # Imports for References for supportingPatientInformation
    # doseNumberPositiveInt (positiveInt)
    from spark_auto_mapper_fhir.fhir_types.positive_int import FhirPositiveInt

    # doseNumberString (string)
    # seriesDosesPositiveInt (positiveInt)
    # seriesDosesString (string)


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class ImmunizationRecommendationRecommendation(FhirBackboneElementBase):
    """
    ImmunizationRecommendation.Recommendation
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        vaccineCode: Optional[
            FhirList[CodeableConcept[VaccineAdministeredValueSetCode]]
        ] = None,
        targetDisease: Optional[
            CodeableConcept[ImmunizationRecommendationTargetDiseaseCodesCode]
        ] = None,
        contraindicatedVaccineCode: Optional[
            FhirList[CodeableConcept[VaccineAdministeredValueSetCode]]
        ] = None,
        forecastStatus: CodeableConcept[ImmunizationRecommendationStatusCodesCode],
        forecastReason: Optional[
            FhirList[CodeableConcept[ImmunizationRecommendationReasonCodesCode]]
        ] = None,
        dateCriterion: Optional[
            FhirList[ImmunizationRecommendationDateCriterion]
        ] = None,
        description: Optional[FhirString] = None,
        series: Optional[FhirString] = None,
        supportingImmunization: Optional[
            FhirList[Reference[Union[Immunization, ImmunizationEvaluation]]]
        ] = None,
        supportingPatientInformation: Optional[
            FhirList[Reference[Union[Resource]]]
        ] = None,
        doseNumberPositiveInt: Optional[FhirPositiveInt] = None,
        doseNumberString: Optional[FhirString] = None,
        seriesDosesPositiveInt: Optional[FhirPositiveInt] = None,
        seriesDosesString: Optional[FhirString] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param vaccineCode: Vaccine(s) or vaccine group that pertain to the recommendation.
            :param targetDisease: The targeted disease for the recommendation.
            :param contraindicatedVaccineCode: Vaccine(s) which should not be used to fulfill the recommendation.
            :param forecastStatus: Indicates the patient status with respect to the path to immunity for the
        target disease.
            :param forecastReason: The reason for the assigned forecast status.
            :param dateCriterion: Vaccine date recommendations.  For example, earliest date to administer,
        latest date to administer, etc.
            :param description: Contains the description about the protocol under which the vaccine was
        administered.
            :param series: One possible path to achieve presumed immunity against a disease - within the
        context of an authority.
            :param supportingImmunization: Immunization event history and/or evaluation that supports the status and
        recommendation.
            :param supportingPatientInformation: Patient Information that supports the status and recommendation.  This
        includes patient observations, adverse reactions and allergy/intolerance
        information.
            :param doseNumberPositiveInt: None
            :param doseNumberString: None
            :param seriesDosesPositiveInt: None
            :param seriesDosesString: None
        """
        super().__init__(
            id_=id_,
            extension=extension,
            vaccineCode=vaccineCode,
            targetDisease=targetDisease,
            contraindicatedVaccineCode=contraindicatedVaccineCode,
            forecastStatus=forecastStatus,
            forecastReason=forecastReason,
            dateCriterion=dateCriterion,
            description=description,
            series=series,
            supportingImmunization=supportingImmunization,
            supportingPatientInformation=supportingPatientInformation,
            doseNumberPositiveInt=doseNumberPositiveInt,
            doseNumberString=doseNumberString,
            seriesDosesPositiveInt=seriesDosesPositiveInt,
            seriesDosesString=seriesDosesString,
        )
