from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # type_ (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for type_
    # Import for CodeableConcept for type_
    from spark_auto_mapper_fhir.value_sets.generic_type import GenericTypeCode

    # End Import for CodeableConcept for type_
    # applicability (CodeableConcept)
    # End Import for References for applicability
    # Import for CodeableConcept for applicability
    from spark_auto_mapper_fhir.value_sets.benefit_cost_applicability import (
        BenefitCostApplicabilityCode,
    )

    # End Import for CodeableConcept for applicability
    # qualifiers (CodeableConcept)
    # End Import for References for qualifiers
    # Import for CodeableConcept for qualifiers
    # End Import for CodeableConcept for qualifiers
    # value (Quantity)
    from spark_auto_mapper_fhir.complex_types.quantity import Quantity


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class InsurancePlanCost(FhirBackboneElementBase):
    """
    InsurancePlan.Cost
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        type_: CodeableConcept[GenericTypeCode],
        applicability: Optional[CodeableConcept[BenefitCostApplicabilityCode]] = None,
        qualifiers: Optional[FhirList[CodeableConcept[GenericTypeCode]]] = None,
        value: Optional[Quantity] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param type_: Type of cost (copay; individual cap; family cap; coinsurance; deductible).
            :param applicability: Whether the cost applies to in-network or out-of-network providers (in-
        network; out-of-network; other).
            :param qualifiers: Additional information about the cost, such as information about funding
        sources (e.g. HSA, HRA, FSA, RRA).
            :param value: The actual cost value. (some of the costs may be represented as percentages
        rather than currency, e.g. 10% coinsurance).
        """
        super().__init__(
            id_=id_,
            extension=extension,
            type_=type_,
            applicability=applicability,
            qualifiers=qualifiers,
            value=value,
        )
