from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # function (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for function
    # Import for CodeableConcept for function
    from spark_auto_mapper_fhir.value_sets.medication_dispense_performer_function_codes import (
        MedicationDispensePerformerFunctionCodesCode,
    )

    # End Import for CodeableConcept for function
    # actor (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for actor
    from spark_auto_mapper_fhir.resources.practitioner import Practitioner
    from spark_auto_mapper_fhir.resources.practitioner_role import PractitionerRole
    from spark_auto_mapper_fhir.resources.organization import Organization
    from spark_auto_mapper_fhir.resources.patient import Patient
    from spark_auto_mapper_fhir.resources.device import Device
    from spark_auto_mapper_fhir.resources.related_person import RelatedPerson


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class MedicationDispensePerformer(FhirBackboneElementBase):
    """
    MedicationDispense.Performer
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        function: Optional[
            CodeableConcept[MedicationDispensePerformerFunctionCodesCode]
        ] = None,
        actor: Reference[
            Union[
                Practitioner,
                PractitionerRole,
                Organization,
                Patient,
                Device,
                RelatedPerson,
            ]
        ],
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param function: Distinguishes the type of performer in the dispense.  For example, date
        enterer, packager, final checker.
            :param actor: The device, practitioner, etc. who performed the action.  It should be assumed
        that the actor is the dispenser of the medication.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            function=function,
            actor=actor,
        )
