from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # code (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for code
    # Import for CodeableConcept for code
    from spark_auto_mapper_fhir.value_sets.generic_type import GenericTypeCode

    # End Import for CodeableConcept for code
    # firstDose (Quantity)
    from spark_auto_mapper_fhir.complex_types.quantity import Quantity

    # maxSingleDose (Quantity)
    # maxDosePerDay (Quantity)
    # maxDosePerTreatmentPeriod (Ratio)
    from spark_auto_mapper_fhir.complex_types.ratio import Ratio

    # maxTreatmentPeriod (Duration)
    from spark_auto_mapper_fhir.complex_types.duration import Duration

    # targetSpecies (MedicinalProductPharmaceutical.TargetSpecies)
    from spark_auto_mapper_fhir.backbone_elements.medicinal_product_pharmaceutical_target_species import (
        MedicinalProductPharmaceuticalTargetSpecies,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class MedicinalProductPharmaceuticalRouteOfAdministration(FhirBackboneElementBase):
    """
    MedicinalProductPharmaceutical.RouteOfAdministration
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        code: CodeableConcept[GenericTypeCode],
        firstDose: Optional[Quantity] = None,
        maxSingleDose: Optional[Quantity] = None,
        maxDosePerDay: Optional[Quantity] = None,
        maxDosePerTreatmentPeriod: Optional[Ratio] = None,
        maxTreatmentPeriod: Optional[Duration] = None,
        targetSpecies: Optional[
            FhirList[MedicinalProductPharmaceuticalTargetSpecies]
        ] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param code: Coded expression for the route.
            :param firstDose: The first dose (dose quantity) administered in humans can be specified, for a
        product under investigation, using a numerical value and its unit of
        measurement.
            :param maxSingleDose: The maximum single dose that can be administered as per the protocol of a
        clinical trial can be specified using a numerical value and its unit of
        measurement.
            :param maxDosePerDay: The maximum dose per day (maximum dose quantity to be administered in any one
        24-h period) that can be administered as per the protocol referenced in the
        clinical trial authorisation.
            :param maxDosePerTreatmentPeriod: The maximum dose per treatment period that can be administered as per the
        protocol referenced in the clinical trial authorisation.
            :param maxTreatmentPeriod: The maximum treatment period during which an Investigational Medicinal Product
        can be administered as per the protocol referenced in the clinical trial
        authorisation.
            :param targetSpecies: A species for which this route applies.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            code=code,
            firstDose=firstDose,
            maxSingleDose=maxSingleDose,
            maxDosePerDay=maxDosePerDay,
            maxDosePerTreatmentPeriod=maxDosePerTreatmentPeriod,
            maxTreatmentPeriod=maxTreatmentPeriod,
            targetSpecies=targetSpecies,
        )
