from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId
from spark_auto_mapper_fhir.fhir_types.uri import FhirUri

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # type_ (repositoryType)
    from spark_auto_mapper_fhir.value_sets.repository_type import RepositoryTypeCode

    # url (uri)
    # name (string)
    # datasetId (string)
    # variantsetId (string)
    # readsetId (string)


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class MolecularSequenceRepository(FhirBackboneElementBase):
    """
    MolecularSequence.Repository
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        type_: RepositoryTypeCode,
        url: Optional[FhirUri] = None,
        name: Optional[FhirString] = None,
        datasetId: Optional[FhirString] = None,
        variantsetId: Optional[FhirString] = None,
        readsetId: Optional[FhirString] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param type_: Click and see / RESTful API / Need login to see / RESTful API with
        authentication / Other ways to see resource.
            :param url: URI of an external repository which contains further details about the
        genetics data.
            :param name: URI of an external repository which contains further details about the
        genetics data.
            :param datasetId: Id of the variant in this external repository. The server will understand how
        to use this id to call for more info about datasets in external repository.
            :param variantsetId: Id of the variantset in this external repository. The server will understand
        how to use this id to call for more info about variantsets in external
        repository.
            :param readsetId: Id of the read in this external repository.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            type_=type_,
            url=url,
            name=name,
            datasetId=datasetId,
            variantsetId=variantsetId,
            readsetId=readsetId,
        )
