from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId
from spark_auto_mapper_fhir.resources.resource import Resource

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # role (ProvenanceEntityRole)
    from spark_auto_mapper_fhir.value_sets.provenance_entity_role import (
        ProvenanceEntityRoleCode,
    )

    # what (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for what
    # agent (Provenance.Agent)
    from spark_auto_mapper_fhir.backbone_elements.provenance_agent import (
        ProvenanceAgent,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class ProvenanceEntity(FhirBackboneElementBase):
    """
    Provenance.Entity
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        role: ProvenanceEntityRoleCode,
        what: Reference[Union[Resource]],
        agent: Optional[FhirList[ProvenanceAgent]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param role: How the entity was used during the activity.
            :param what: Identity of the  Entity used. May be a logical or physical uri and maybe
        absolute or relative.
            :param agent: The entity is attributed to an agent to express the agent's responsibility for
        that entity, possibly along with other agents. This description can be
        understood as shorthand for saying that the agent was responsible for the
        activity which generated the entity.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            role=role,
            what=what,
            agent=agent,
        )
