from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.integer import FhirInteger
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # description (string)
    # type_ (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for type_
    # Import for CodeableConcept for type_
    from spark_auto_mapper_fhir.value_sets.risk_estimate_type import (
        RiskEstimateTypeCode,
    )

    # End Import for CodeableConcept for type_
    # value (decimal)
    from spark_auto_mapper_fhir.fhir_types.decimal import FhirDecimal

    # unitOfMeasure (CodeableConcept)
    # End Import for References for unitOfMeasure
    # Import for CodeableConcept for unitOfMeasure
    from spark_auto_mapper_fhir.value_sets.ucum_codes import UCUMCodesCode

    # End Import for CodeableConcept for unitOfMeasure
    # denominatorCount (integer)
    # numeratorCount (integer)
    # precisionEstimate (RiskEvidenceSynthesis.PrecisionEstimate)
    from spark_auto_mapper_fhir.backbone_elements.risk_evidence_synthesis_precision_estimate import (
        RiskEvidenceSynthesisPrecisionEstimate,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class RiskEvidenceSynthesisRiskEstimate(FhirBackboneElementBase):
    """
    RiskEvidenceSynthesis.RiskEstimate
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        description: Optional[FhirString] = None,
        type_: Optional[CodeableConcept[RiskEstimateTypeCode]] = None,
        value: Optional[FhirDecimal] = None,
        unitOfMeasure: Optional[CodeableConcept[UCUMCodesCode]] = None,
        denominatorCount: Optional[FhirInteger] = None,
        numeratorCount: Optional[FhirInteger] = None,
        precisionEstimate: Optional[
            FhirList[RiskEvidenceSynthesisPrecisionEstimate]
        ] = None,
    ) -> None:
        """

        :param id_: id of resource
        :param extension: extensions
        :param description: Human-readable summary of risk estimate.
        :param type_: Examples include proportion and mean.
        :param value: The point estimate of the risk estimate.
        :param unitOfMeasure: Specifies the UCUM unit for the outcome.
        :param denominatorCount: The sample size for the group that was measured for this risk estimate.
        :param numeratorCount: The number of group members with the outcome of interest.
        :param precisionEstimate: A description of the precision of the estimate for the effect.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            description=description,
            type_=type_,
            value=value,
            unitOfMeasure=unitOfMeasure,
            denominatorCount=denominatorCount,
            numeratorCount=numeratorCount,
            precisionEstimate=precisionEstimate,
        )
