from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # material (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # End Import for References for material
    # Import for CodeableConcept for material
    from spark_auto_mapper_fhir.value_sets.container_materials import (
        ContainerMaterialsCode,
    )

    # End Import for CodeableConcept for material
    # type_ (CodeableConcept)
    # End Import for References for type_
    # Import for CodeableConcept for type_
    from spark_auto_mapper_fhir.value_sets.specimen_container_type import (
        SpecimenContainerTypeCode,
    )

    # End Import for CodeableConcept for type_
    # cap (CodeableConcept)
    # End Import for References for cap
    # Import for CodeableConcept for cap
    from spark_auto_mapper_fhir.value_sets.container_cap import ContainerCapCode

    # End Import for CodeableConcept for cap
    # description (string)
    # capacity (Quantity)
    from spark_auto_mapper_fhir.complex_types.quantity import Quantity

    # additive (SpecimenDefinition.Additive)
    from spark_auto_mapper_fhir.backbone_elements.specimen_definition_additive import (
        SpecimenDefinitionAdditive,
    )

    # preparation (string)
    # minimumVolumeQuantity (Quantity)
    # minimumVolumeString (string)


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class SpecimenDefinitionContainer(FhirBackboneElementBase):
    """
    SpecimenDefinition.Container
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        material: Optional[CodeableConcept[ContainerMaterialsCode]] = None,
        type_: Optional[CodeableConcept[SpecimenContainerTypeCode]] = None,
        cap: Optional[CodeableConcept[ContainerCapCode]] = None,
        description: Optional[FhirString] = None,
        capacity: Optional[Quantity] = None,
        additive: Optional[FhirList[SpecimenDefinitionAdditive]] = None,
        preparation: Optional[FhirString] = None,
        minimumVolumeQuantity: Optional[Quantity] = None,
        minimumVolumeString: Optional[FhirString] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param material: The type of material of the container.
            :param type_: The type of container used to contain this kind of specimen.
            :param cap: Color of container cap.
            :param description: The textual description of the kind of container.
            :param capacity: The capacity (volume or other measure) of this kind of container.
            :param additive: Substance introduced in the kind of container to preserve, maintain or enhance
        the specimen. Examples: Formalin, Citrate, EDTA.
            :param preparation: Special processing that should be applied to the container for this kind of
        specimen.
            :param minimumVolumeQuantity: None
            :param minimumVolumeString: None
        """
        super().__init__(
            id_=id_,
            extension=extension,
            material=material,
            type_=type_,
            cap=cap,
            description=description,
            capacity=capacity,
            additive=additive,
            preparation=preparation,
            minimumVolumeQuantity=minimumVolumeQuantity,
            minimumVolumeString=minimumVolumeString,
        )
