from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_backbone_element_base import (
    FhirBackboneElementBase,
)

if TYPE_CHECKING:
    pass
    # count (positiveInt)
    from spark_auto_mapper_fhir.fhir_types.positive_int import FhirPositiveInt

    # countMax (positiveInt)
    # duration (decimal)
    from spark_auto_mapper_fhir.fhir_types.decimal import FhirDecimal

    # durationMax (decimal)
    # durationUnit (UnitsOfTime)
    from spark_auto_mapper_fhir.value_sets.units_of_time import UnitsOfTimeCode

    # frequency (positiveInt)
    # frequencyMax (positiveInt)
    # period (decimal)
    # periodMax (decimal)
    # periodUnit (UnitsOfTime)
    # dayOfWeek (DaysOfWeek)
    from spark_auto_mapper_fhir.value_sets.days_of_week import DaysOfWeekCode

    # timeOfDay (time)
    from spark_auto_mapper_fhir.fhir_types.time import FhirTime

    # when (EventTiming)
    from spark_auto_mapper_fhir.value_sets.event_timing import EventTimingCode

    # offset (unsignedInt)
    from spark_auto_mapper_fhir.complex_types.unsigned_int import unsignedInt

    # boundsDuration (Duration)
    from spark_auto_mapper_fhir.complex_types.duration import Duration

    # boundsRange (Range)
    from spark_auto_mapper_fhir.complex_types.range import Range

    # boundsPeriod (Period)
    from spark_auto_mapper_fhir.complex_types.period import Period


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class TimingRepeat(FhirBackboneElementBase):
    """
    Timing.Repeat
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        count: Optional[FhirPositiveInt] = None,
        countMax: Optional[FhirPositiveInt] = None,
        duration: Optional[FhirDecimal] = None,
        durationMax: Optional[FhirDecimal] = None,
        durationUnit: Optional[UnitsOfTimeCode] = None,
        frequency: Optional[FhirPositiveInt] = None,
        frequencyMax: Optional[FhirPositiveInt] = None,
        period: Optional[FhirDecimal] = None,
        periodMax: Optional[FhirDecimal] = None,
        periodUnit: Optional[UnitsOfTimeCode] = None,
        dayOfWeek: Optional[FhirList[DaysOfWeekCode]] = None,
        timeOfDay: Optional[FhirList[FhirTime]] = None,
        when: Optional[FhirList[EventTimingCode]] = None,
        offset: Optional[unsignedInt] = None,
        boundsDuration: Optional[Duration] = None,
        boundsRange: Optional[Range] = None,
        boundsPeriod: Optional[Period] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param count: A total count of the desired number of repetitions across the duration of the
        entire timing specification. If countMax is present, this element indicates
        the lower bound of the allowed range of count values.
            :param countMax: If present, indicates that the count is a range - so to perform the action
        between [count] and [countMax] times.
            :param duration: How long this thing happens for when it happens. If durationMax is present,
        this element indicates the lower bound of the allowed range of the duration.
            :param durationMax: If present, indicates that the duration is a range - so to perform the action
        between [duration] and [durationMax] time length.
            :param durationUnit: The units of time for the duration, in UCUM units.
            :param frequency: The number of times to repeat the action within the specified period. If
        frequencyMax is present, this element indicates the lower bound of the allowed
        range of the frequency.
            :param frequencyMax: If present, indicates that the frequency is a range - so to repeat between
        [frequency] and [frequencyMax] times within the period or period range.
            :param period: Indicates the duration of time over which repetitions are to occur; e.g. to
        express "3 times per day", 3 would be the frequency and "1 day" would be the
        period. If periodMax is present, this element indicates the lower bound of the
        allowed range of the period length.
            :param periodMax: If present, indicates that the period is a range from [period] to [periodMax],
        allowing expressing concepts such as "do this once every 3-5 days.
            :param periodUnit: The units of time for the period in UCUM units.
            :param dayOfWeek: If one or more days of week is provided, then the action happens only on the
        specified day(s).
            :param timeOfDay: Specified time of day for action to take place.
            :param when: An approximate time period during the day, potentially linked to an event of
        daily living that indicates when the action should occur.
            :param offset: The number of minutes from the event. If the event code does not indicate
        whether the minutes is before or after the event, then the offset is assumed
        to be after the event.
            :param boundsDuration: None
            :param boundsRange: None
            :param boundsPeriod: None
        """
        super().__init__(
            id_=id_,
            extension=extension,
            count=count,
            countMax=countMax,
            duration=duration,
            durationMax=durationMax,
            durationUnit=durationUnit,
            frequency=frequency,
            frequencyMax=frequencyMax,
            period=period,
            periodMax=periodMax,
            periodUnit=periodUnit,
            dayOfWeek=dayOfWeek,
            timeOfDay=timeOfDay,
            when=when,
            offset=offset,
            boundsDuration=boundsDuration,
            boundsRange=boundsRange,
            boundsPeriod=boundsPeriod,
        )
