from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString

from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_complex_type_base import FhirComplexTypeBase

if TYPE_CHECKING:
    pass
    # use (AddressUse)
    from spark_auto_mapper_fhir.value_sets.address_use import AddressUseCode

    # type_ (AddressType)
    from spark_auto_mapper_fhir.value_sets.address_type import AddressTypeCode

    # text (string)
    # line (string)
    # city (string)
    # district (string)
    # state (string)
    # postalCode (string)
    # country (string)
    # period (Period)
    from spark_auto_mapper_fhir.complex_types.period import Period


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class Address(FhirComplexTypeBase):
    """
    Address
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        use: Optional[AddressUseCode] = None,
        type_: Optional[AddressTypeCode] = None,
        text: Optional[FhirString] = None,
        line: Optional[FhirList[FhirString]] = None,
        city: Optional[FhirString] = None,
        district: Optional[FhirString] = None,
        state: Optional[FhirString] = None,
        postalCode: Optional[FhirString] = None,
        country: Optional[FhirString] = None,
        period: Optional[Period] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param use: The purpose of this address.
            :param type_: Distinguishes between physical addresses (those you can visit) and mailing
        addresses (e.g. PO Boxes and care-of addresses). Most addresses are both.
            :param text: Specifies the entire address as it should be displayed e.g. on a postal label.
        This may be provided instead of or as well as the specific parts.
            :param line: This component contains the house number, apartment number, street name,
        street direction,  P.O. Box number, delivery hints, and similar address
        information.
            :param city: The name of the city, town, suburb, village or other community or delivery
        center.
            :param district: The name of the administrative area (county).
            :param state: Sub-unit of a country with limited sovereignty in a federally organized
        country. A code may be used if codes are in common use (e.g. US 2 letter state
        codes).
            :param postalCode: A postal code designating a region defined by the postal service.
            :param country: Country - a nation as commonly understood or generally accepted.
            :param period: Time period when address was/is in use.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            use=use,
            type_=type_,
            text=text,
            line=line,
            city=city,
            district=district,
            state=state,
            postalCode=postalCode,
            country=country,
            period=period,
        )
