from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.boolean import FhirBoolean
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.fhir_types.uri import FhirUri

from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_complex_type_base import FhirComplexTypeBase

if TYPE_CHECKING:
    pass
    # system (uri)
    # version (string)
    # code (code)
    # display (string)
    # userSelected (boolean)


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
from typing import TypeVar, Generic
from spark_auto_mapper_fhir.base_types.FhirValueSetBase import FhirValueSetBase

_T = TypeVar("_T", bound=Union[FhirValueSetBase])


class Coding(FhirComplexTypeBase, Generic[_T]):
    """
    Coding
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        system: Optional[FhirUri] = None,
        version: Optional[FhirString] = None,
        code: Optional[_T] = None,
        display: Optional[FhirString] = None,
        userSelected: Optional[FhirBoolean] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param system: The identification of the code system that defines the meaning of the symbol
        in the code.
            :param version: The version of the code system which was used when choosing this code. Note
        that a well-maintained code system does not need the version reported, because
        the meaning of codes is consistent across versions. However this cannot
        consistently be assured, and when the meaning is not guaranteed to be
        consistent, the version SHOULD be exchanged.
            :param code: A symbol in syntax defined by the system. The symbol may be a predefined code
        or an expression in a syntax defined by the coding system (e.g. post-
        coordination).
            :param display: A representation of the meaning of the code in the system, following the rules
        of the system.
            :param userSelected: Indicates that this coding was chosen by a user directly - e.g. off a pick
        list of available items (codes or displays).
        """
        super().__init__(
            id_=id_,
            extension=extension,
            system=system,
            version=version,
            code=code,
            display=display,
            userSelected=userSelected,
        )
