from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString

from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_complex_type_base import FhirComplexTypeBase

if TYPE_CHECKING:
    pass
    # use (NameUse)
    from spark_auto_mapper_fhir.value_sets.name_use import NameUseCode

    # text (string)
    # family (string)
    # given (string)
    # prefix (string)
    # suffix (string)
    # period (Period)
    from spark_auto_mapper_fhir.complex_types.period import Period


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class HumanName(FhirComplexTypeBase):
    """
    HumanName
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        use: Optional[NameUseCode] = None,
        text: Optional[FhirString] = None,
        family: Optional[FhirString] = None,
        given: Optional[FhirList[FhirString]] = None,
        prefix: Optional[FhirList[FhirString]] = None,
        suffix: Optional[FhirList[FhirString]] = None,
        period: Optional[Period] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param use: Identifies the purpose for this name.
            :param text: Specifies the entire name as it should be displayed e.g. on an application UI.
        This may be provided instead of or as well as the specific parts.
            :param family: The part of a name that links to the genealogy. In some cultures (e.g.
        Eritrea) the family name of a son is the first name of his father.
            :param given: Given name.
            :param prefix: Part of the name that is acquired as a title due to academic, legal,
        employment or nobility status, etc. and that appears at the start of the name.
            :param suffix: Part of the name that is acquired as a title due to academic, legal,
        employment or nobility status, etc. and that appears at the end of the name.
            :param period: Indicates the period of time when this name was valid for the named person.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            use=use,
            text=text,
            family=family,
            given=given,
            prefix=prefix,
            suffix=suffix,
            period=period,
        )
