from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.fhir_types.uri import FhirUri

from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_complex_type_base import FhirComplexTypeBase

if TYPE_CHECKING:
    pass
    # use (IdentifierUse)
    from spark_auto_mapper_fhir.value_sets.identifier_use import IdentifierUseCode

    # type_ (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # Import for CodeableConcept for type_
    from spark_auto_mapper_fhir.value_sets.identifier_type_codes import (
        IdentifierTypeCodesCode,
    )

    # End Import for CodeableConcept for type_
    # system (uri)
    # value (string)
    # period (Period)
    from spark_auto_mapper_fhir.complex_types.period import Period

    # assigner (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for assigner
    from spark_auto_mapper_fhir.resources.organization import Organization


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class Identifier(FhirComplexTypeBase):
    """
    Identifier
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        use: Optional[IdentifierUseCode] = None,
        type_: Optional[CodeableConcept[IdentifierTypeCodesCode]] = None,
        system: Optional[FhirUri] = None,
        value: Optional[FhirString] = None,
        period: Optional[Period] = None,
        assigner: Optional[Reference[Union[Organization]]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param use: The purpose of this identifier.
            :param type_: A coded type for the identifier that can be used to determine which identifier
        to use for a specific purpose.
            :param system: Establishes the namespace for the value - that is, a URL that describes a set
        values that are unique.
            :param value: The portion of the identifier typically relevant to the user and which is
        unique within the context of the system.
            :param period: Time period during which identifier is/was valid for use.
            :param assigner: Organization that issued/manages the identifier.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            use=use,
            type_=type_,
            system=system,
            value=value,
            period=period,
            assigner=assigner,
        )
