from __future__ import annotations
from typing import Optional, TYPE_CHECKING

from spark_auto_mapper_fhir.fhir_types.list import FhirList

from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_complex_type_base import FhirComplexTypeBase

if TYPE_CHECKING:
    pass
    # origin (Quantity)
    from spark_auto_mapper_fhir.complex_types.quantity import Quantity

    # period (decimal)
    from spark_auto_mapper_fhir.fhir_types.decimal import FhirDecimal

    # factor (decimal)
    # lowerLimit (decimal)
    # upperLimit (decimal)
    # dimensions (positiveInt)
    from spark_auto_mapper_fhir.fhir_types.positive_int import FhirPositiveInt

    # data (SampledDataDataType)
    from spark_auto_mapper_fhir.complex_types.sampled_data_data_type import (
        SampledDataDataType,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class SampledData(FhirComplexTypeBase):
    """
    SampledData
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: Optional[FhirId] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        origin: Quantity,
        period: FhirDecimal,
        factor: Optional[FhirDecimal] = None,
        lowerLimit: Optional[FhirDecimal] = None,
        upperLimit: Optional[FhirDecimal] = None,
        dimensions: FhirPositiveInt,
        data: Optional[SampledDataDataType] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param extension: extensions
            :param origin: The base quantity that a measured value of zero represents. In addition, this
        provides the units of the entire measurement series.
            :param period: The length of time between sampling times, measured in milliseconds.
            :param factor: A correction factor that is applied to the sampled data points before they are
        added to the origin.
            :param lowerLimit: The lower limit of detection of the measured points. This is needed if any of
        the data points have the value "L" (lower than detection limit).
            :param upperLimit: The upper limit of detection of the measured points. This is needed if any of
        the data points have the value "U" (higher than detection limit).
            :param dimensions: The number of sample points at each time point. If this value is greater than
        one, then the dimensions will be interlaced - all the sample points for a
        point in time will be recorded at once.
            :param data: A series of data points which are decimal values separated by a single space
        (character u20). The special values "E" (error), "L" (below detection limit)
        and "U" (above detection limit) can also be used in place of a decimal value.
        """
        super().__init__(
            id_=id_,
            extension=extension,
            origin=origin,
            period=period,
            factor=factor,
            lowerLimit=lowerLimit,
            upperLimit=upperLimit,
            dimensions=dimensions,
            data=data,
        )
