from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

# noinspection PyPackageRequirements
from pyspark.sql.types import StructType, DataType
from spark_auto_mapper_fhir.fhir_types.boolean import FhirBoolean
from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.complex_types.meta import Meta
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_resource_base import FhirResourceBase
from spark_fhir_schemas.r4.resources.catalogentry import CatalogEntrySchema

if TYPE_CHECKING:
    pass
    # identifier (Identifier)
    from spark_auto_mapper_fhir.complex_types.identifier import Identifier

    # type_ (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # Import for CodeableConcept for type_
    from spark_auto_mapper_fhir.value_sets.generic_type import GenericTypeCode

    # End Import for CodeableConcept for type_
    # orderable (boolean)
    # referencedItem (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for referencedItem
    from spark_auto_mapper_fhir.resources.medication import Medication
    from spark_auto_mapper_fhir.resources.device import Device
    from spark_auto_mapper_fhir.resources.organization import Organization
    from spark_auto_mapper_fhir.resources.practitioner import Practitioner
    from spark_auto_mapper_fhir.resources.practitioner_role import PractitionerRole
    from spark_auto_mapper_fhir.resources.healthcare_service import HealthcareService
    from spark_auto_mapper_fhir.resources.activity_definition import ActivityDefinition
    from spark_auto_mapper_fhir.resources.plan_definition import PlanDefinition
    from spark_auto_mapper_fhir.resources.specimen_definition import SpecimenDefinition
    from spark_auto_mapper_fhir.resources.observation_definition import (
        ObservationDefinition,
    )
    from spark_auto_mapper_fhir.resources.binary import Binary

    # additionalIdentifier (Identifier)
    # classification (CodeableConcept)
    # Import for CodeableConcept for classification
    # End Import for CodeableConcept for classification
    # status (PublicationStatus)
    from spark_auto_mapper_fhir.value_sets.publication_status import (
        PublicationStatusCode,
    )

    # validityPeriod (Period)
    from spark_auto_mapper_fhir.complex_types.period import Period

    # validTo (dateTime)
    # lastUpdated (dateTime)
    # additionalCharacteristic (CodeableConcept)
    # Import for CodeableConcept for additionalCharacteristic
    # End Import for CodeableConcept for additionalCharacteristic
    # additionalClassification (CodeableConcept)
    # Import for CodeableConcept for additionalClassification
    # End Import for CodeableConcept for additionalClassification
    # relatedEntry (CatalogEntry.RelatedEntry)
    from spark_auto_mapper_fhir.backbone_elements.catalog_entry_related_entry import (
        CatalogEntryRelatedEntry,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class CatalogEntry(FhirResourceBase):
    """
    CatalogEntry
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: FhirId,
        meta: Optional[Meta] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        identifier: Optional[FhirList[Identifier]] = None,
        type_: Optional[CodeableConcept[GenericTypeCode]] = None,
        orderable: FhirBoolean,
        referencedItem: Reference[
            Union[
                Medication,
                Device,
                Organization,
                Practitioner,
                PractitionerRole,
                HealthcareService,
                ActivityDefinition,
                PlanDefinition,
                SpecimenDefinition,
                ObservationDefinition,
                Binary,
            ]
        ],
        additionalIdentifier: Optional[FhirList[Identifier]] = None,
        classification: Optional[FhirList[CodeableConcept[GenericTypeCode]]] = None,
        status: Optional[PublicationStatusCode] = None,
        validityPeriod: Optional[Period] = None,
        validTo: Optional[FhirDateTime] = None,
        lastUpdated: Optional[FhirDateTime] = None,
        additionalCharacteristic: Optional[
            FhirList[CodeableConcept[GenericTypeCode]]
        ] = None,
        additionalClassification: Optional[
            FhirList[CodeableConcept[GenericTypeCode]]
        ] = None,
        relatedEntry: Optional[FhirList[CatalogEntryRelatedEntry]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param meta: Meta
            :param extension: extensions
            :param identifier: Used in supporting different identifiers for the same product, e.g.
        manufacturer code and retailer code.
            :param type_: The type of item - medication, device, service, protocol or other.
            :param orderable: Whether the entry represents an orderable item.
            :param referencedItem: The item in a catalog or definition.
            :param additionalIdentifier: Used in supporting related concepts, e.g. NDC to RxNorm.
            :param classification: Classes of devices, or ATC for medication.
            :param status: Used to support catalog exchange even for unsupported products, e.g. getting
        list of medications even if not prescribable.
            :param validityPeriod: The time period in which this catalog entry is expected to be active.
            :param validTo: The date until which this catalog entry is expected to be active.
            :param lastUpdated: Typically date of issue is different from the beginning of the validity. This
        can be used to see when an item was last updated.
            :param additionalCharacteristic: Used for examplefor Out of Formulary, or any specifics.
            :param additionalClassification: User for example for ATC classification, or.
            :param relatedEntry: Used for example, to point to a substance, or to a device used to administer a
        medication.
        """
        super().__init__(
            resourceType="CatalogEntry",
            id_=id_,
            meta=meta,
            extension=extension,
            identifier=identifier,
            type_=type_,
            orderable=orderable,
            referencedItem=referencedItem,
            additionalIdentifier=additionalIdentifier,
            classification=classification,
            status=status,
            validityPeriod=validityPeriod,
            validTo=validTo,
            lastUpdated=lastUpdated,
            additionalCharacteristic=additionalCharacteristic,
            additionalClassification=additionalClassification,
            relatedEntry=relatedEntry,
        )

    def get_schema(
        self, include_extension: bool
    ) -> Optional[Union[StructType, DataType]]:
        return CatalogEntrySchema.get_schema(include_extension=include_extension)
