from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

# noinspection PyPackageRequirements
from pyspark.sql.types import StructType, DataType
from spark_auto_mapper_fhir.fhir_types.boolean import FhirBoolean
from spark_auto_mapper_fhir.fhir_types.date import FhirDate
from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.complex_types.meta import Meta
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId
from spark_auto_mapper_fhir.fhir_types.uri import FhirUri

from spark_auto_mapper_fhir.base_types.fhir_resource_base import FhirResourceBase
from spark_fhir_schemas.r4.resources.chargeitemdefinition import (
    ChargeItemDefinitionSchema,
)

if TYPE_CHECKING:
    pass
    # url (uri)
    # identifier (Identifier)
    from spark_auto_mapper_fhir.complex_types.identifier import Identifier

    # version (string)
    # title (string)
    # derivedFromUri (uri)
    # partOf (canonical)
    from spark_auto_mapper_fhir.fhir_types.canonical import FhirCanonical

    # replaces (canonical)
    # status (PublicationStatus)
    from spark_auto_mapper_fhir.value_sets.publication_status import (
        PublicationStatusCode,
    )

    # experimental (boolean)
    # date (dateTime)
    # publisher (string)
    # contact (ContactDetail)
    from spark_auto_mapper_fhir.complex_types.contact_detail import ContactDetail

    # description (markdown)
    from spark_auto_mapper_fhir.fhir_types.markdown import FhirMarkdown

    # useContext (UsageContext)
    from spark_auto_mapper_fhir.complex_types.usage_context import UsageContext

    # jurisdiction (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # Import for CodeableConcept for jurisdiction
    from spark_auto_mapper_fhir.value_sets.jurisdiction_value_set import (
        JurisdictionValueSetCode,
    )

    # End Import for CodeableConcept for jurisdiction
    # copyright (markdown)
    # approvalDate (date)
    # lastReviewDate (date)
    # effectivePeriod (Period)
    from spark_auto_mapper_fhir.complex_types.period import Period

    # code (CodeableConcept)
    # Import for CodeableConcept for code
    from spark_auto_mapper_fhir.value_sets.charge_item_code import ChargeItemCodeCode

    # End Import for CodeableConcept for code
    # instance (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for instance
    from spark_auto_mapper_fhir.resources.medication import Medication
    from spark_auto_mapper_fhir.resources.substance import Substance
    from spark_auto_mapper_fhir.resources.device import Device

    # applicability (ChargeItemDefinition.Applicability)
    from spark_auto_mapper_fhir.backbone_elements.charge_item_definition_applicability import (
        ChargeItemDefinitionApplicability,
    )

    # propertyGroup (ChargeItemDefinition.PropertyGroup)
    from spark_auto_mapper_fhir.backbone_elements.charge_item_definition_property_group import (
        ChargeItemDefinitionPropertyGroup,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class ChargeItemDefinition(FhirResourceBase):
    """
    ChargeItemDefinition
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: FhirId,
        meta: Optional[Meta] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        url: FhirUri,
        identifier: Optional[FhirList[Identifier]] = None,
        version: Optional[FhirString] = None,
        title: Optional[FhirString] = None,
        derivedFromUri: Optional[FhirList[FhirUri]] = None,
        partOf: Optional[FhirList[FhirCanonical]] = None,
        replaces: Optional[FhirList[FhirCanonical]] = None,
        status: PublicationStatusCode,
        experimental: Optional[FhirBoolean] = None,
        date: Optional[FhirDateTime] = None,
        publisher: Optional[FhirString] = None,
        contact: Optional[FhirList[ContactDetail]] = None,
        description: Optional[FhirMarkdown] = None,
        useContext: Optional[FhirList[UsageContext]] = None,
        jurisdiction: Optional[
            FhirList[CodeableConcept[JurisdictionValueSetCode]]
        ] = None,
        copyright: Optional[FhirMarkdown] = None,
        approvalDate: Optional[FhirDate] = None,
        lastReviewDate: Optional[FhirDate] = None,
        effectivePeriod: Optional[Period] = None,
        code: Optional[CodeableConcept[ChargeItemCodeCode]] = None,
        instance: Optional[
            FhirList[Reference[Union[Medication, Substance, Device]]]
        ] = None,
        applicability: Optional[FhirList[ChargeItemDefinitionApplicability]] = None,
        propertyGroup: Optional[FhirList[ChargeItemDefinitionPropertyGroup]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param meta: Meta
            :param extension: extensions
            :param url: An absolute URI that is used to identify this charge item definition when it
        is referenced in a specification, model, design or an instance; also called
        its canonical identifier. This SHOULD be globally unique and SHOULD be a
        literal address at which at which an authoritative instance of this charge
        item definition is (or will be) published. This URL can be the target of a
        canonical reference. It SHALL remain the same when the charge item definition
        is stored on different servers.
            :param identifier: A formal identifier that is used to identify this charge item definition when
        it is represented in other formats, or referenced in a specification, model,
        design or an instance.
            :param version: The identifier that is used to identify this version of the charge item
        definition when it is referenced in a specification, model, design or
        instance. This is an arbitrary value managed by the charge item definition
        author and is not expected to be globally unique. For example, it might be a
        timestamp (e.g. yyyymmdd) if a managed version is not available. There is also
        no expectation that versions can be placed in a lexicographical sequence. To
        provide a version consistent with the Decision Support Service specification,
        use the format Major.Minor.Revision (e.g. 1.0.0). For more information on
        versioning knowledge assets, refer to the Decision Support Service
        specification. Note that a version is required for non-experimental active
        assets.
            :param title: A short, descriptive, user-friendly title for the charge item definition.
            :param derivedFromUri: The URL pointing to an externally-defined charge item definition that is
        adhered to in whole or in part by this definition.
            :param partOf: A larger definition of which this particular definition is a component or
        step.
            :param replaces: As new versions of a protocol or guideline are defined, allows identification
        of what versions are replaced by a new instance.
            :param status: The current state of the ChargeItemDefinition.
            :param experimental: A Boolean value to indicate that this charge item definition is authored for
        testing purposes (or education/evaluation/marketing) and is not intended to be
        used for genuine usage.
            :param date: The date  (and optionally time) when the charge item definition was published.
        The date must change when the business version changes and it must change if
        the status code changes. In addition, it should change when the substantive
        content of the charge item definition changes.
            :param publisher: The name of the organization or individual that published the charge item
        definition.
            :param contact: Contact details to assist a user in finding and communicating with the
        publisher.
            :param description: A free text natural language description of the charge item definition from a
        consumer's perspective.
            :param useContext: The content was developed with a focus and intent of supporting the contexts
        that are listed. These contexts may be general categories (gender, age, ...)
        or may be references to specific programs (insurance plans, studies, ...) and
        may be used to assist with indexing and searching for appropriate charge item
        definition instances.
            :param jurisdiction: A legal or geographic region in which the charge item definition is intended
        to be used.
            :param copyright: A copyright statement relating to the charge item definition and/or its
        contents. Copyright statements are generally legal restrictions on the use and
        publishing of the charge item definition.
            :param approvalDate: The date on which the resource content was approved by the publisher. Approval
        happens once when the content is officially approved for usage.
            :param lastReviewDate: The date on which the resource content was last reviewed. Review happens
        periodically after approval but does not change the original approval date.
            :param effectivePeriod: The period during which the charge item definition content was or is planned
        to be in active use.
            :param code: The defined billing details in this resource pertain to the given billing
        code.
            :param instance: The defined billing details in this resource pertain to the given product
        instance(s).
            :param applicability: Expressions that describe applicability criteria for the billing code.
            :param propertyGroup: Group of properties which are applicable under the same conditions. If no
        applicability rules are established for the group, then all properties always
        apply.
        """
        super().__init__(
            resourceType="ChargeItemDefinition",
            id_=id_,
            meta=meta,
            extension=extension,
            url=url,
            identifier=identifier,
            version=version,
            title=title,
            derivedFromUri=derivedFromUri,
            partOf=partOf,
            replaces=replaces,
            status=status,
            experimental=experimental,
            date=date,
            publisher=publisher,
            contact=contact,
            description=description,
            useContext=useContext,
            jurisdiction=jurisdiction,
            copyright=copyright,
            approvalDate=approvalDate,
            lastReviewDate=lastReviewDate,
            effectivePeriod=effectivePeriod,
            code=code,
            instance=instance,
            applicability=applicability,
            propertyGroup=propertyGroup,
        )

    def get_schema(
        self, include_extension: bool
    ) -> Optional[Union[StructType, DataType]]:
        return ChargeItemDefinitionSchema.get_schema(
            include_extension=include_extension
        )
