from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

# noinspection PyPackageRequirements
from pyspark.sql.types import StructType, DataType
from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.complex_types.meta import Meta
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId
from spark_auto_mapper_fhir.fhir_types.uri import FhirUri

from spark_auto_mapper_fhir.base_types.fhir_resource_base import FhirResourceBase
from spark_fhir_schemas.r4.resources.guidanceresponse import GuidanceResponseSchema

if TYPE_CHECKING:
    pass
    # requestIdentifier (Identifier)
    from spark_auto_mapper_fhir.complex_types.identifier import Identifier

    # identifier (Identifier)
    # status (GuidanceResponseStatus)
    from spark_auto_mapper_fhir.value_sets.guidance_response_status import (
        GuidanceResponseStatusCode,
    )

    # subject (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for subject
    from spark_auto_mapper_fhir.resources.patient import Patient
    from spark_auto_mapper_fhir.resources.group import Group

    # encounter (Reference)
    # Imports for References for encounter
    from spark_auto_mapper_fhir.resources.encounter import Encounter

    # occurrenceDateTime (dateTime)
    # performer (Reference)
    # Imports for References for performer
    from spark_auto_mapper_fhir.resources.device import Device

    # reasonCode (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # Import for CodeableConcept for reasonCode
    from spark_auto_mapper_fhir.value_sets.generic_type import GenericTypeCode

    # End Import for CodeableConcept for reasonCode
    # reasonReference (Reference)
    # Imports for References for reasonReference
    from spark_auto_mapper_fhir.resources.condition import Condition
    from spark_auto_mapper_fhir.resources.observation import Observation
    from spark_auto_mapper_fhir.resources.diagnostic_report import DiagnosticReport
    from spark_auto_mapper_fhir.resources.document_reference import DocumentReference

    # note (Annotation)
    from spark_auto_mapper_fhir.complex_types.annotation import Annotation

    # evaluationMessage (Reference)
    # Imports for References for evaluationMessage
    from spark_auto_mapper_fhir.resources.operation_outcome import OperationOutcome

    # outputParameters (Reference)
    # Imports for References for outputParameters
    from spark_auto_mapper_fhir.resources.parameters import Parameters

    # result (Reference)
    # Imports for References for result
    from spark_auto_mapper_fhir.resources.care_plan import CarePlan
    from spark_auto_mapper_fhir.resources.request_group import RequestGroup

    # dataRequirement (DataRequirement)
    from spark_auto_mapper_fhir.complex_types.data_requirement import DataRequirement

    # moduleUri (uri)
    # moduleCanonical (canonical)
    from spark_auto_mapper_fhir.fhir_types.canonical import FhirCanonical

    # moduleCodeableConcept (CodeableConcept)
    # Import for CodeableConcept for moduleCodeableConcept
    # End Import for CodeableConcept for moduleCodeableConcept


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class GuidanceResponse(FhirResourceBase):
    """
    GuidanceResponse
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: FhirId,
        meta: Optional[Meta] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        requestIdentifier: Optional[Identifier] = None,
        identifier: Optional[FhirList[Identifier]] = None,
        status: GuidanceResponseStatusCode,
        subject: Optional[Reference[Union[Patient, Group]]] = None,
        encounter: Optional[Reference[Union[Encounter]]] = None,
        occurrenceDateTime: Optional[FhirDateTime] = None,
        performer: Optional[Reference[Union[Device]]] = None,
        reasonCode: Optional[FhirList[CodeableConcept[GenericTypeCode]]] = None,
        reasonReference: Optional[
            FhirList[
                Reference[
                    Union[Condition, Observation, DiagnosticReport, DocumentReference]
                ]
            ]
        ] = None,
        note: Optional[FhirList[Annotation]] = None,
        evaluationMessage: Optional[
            FhirList[Reference[Union[OperationOutcome]]]
        ] = None,
        outputParameters: Optional[Reference[Union[Parameters]]] = None,
        result: Optional[Reference[Union[CarePlan, RequestGroup]]] = None,
        dataRequirement: Optional[FhirList[DataRequirement]] = None,
        moduleUri: Optional[FhirUri] = None,
        moduleCanonical: Optional[FhirCanonical] = None,
        moduleCodeableConcept: Optional[CodeableConcept[GenericTypeCode]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param meta: Meta
            :param extension: extensions
            :param requestIdentifier: The identifier of the request associated with this response. If an identifier
        was given as part of the request, it will be reproduced here to enable the
        requester to more easily identify the response in a multi-request scenario.
            :param identifier: Allows a service to provide  unique, business identifiers for the response.
            :param status: The status of the response. If the evaluation is completed successfully, the
        status will indicate success. However, in order to complete the evaluation,
        the engine may require more information. In this case, the status will be
        data-required, and the response will contain a description of the additional
        required information. If the evaluation completed successfully, but the engine
        determines that a potentially more accurate response could be provided if more
        data was available, the status will be data-requested, and the response will
        contain a description of the additional requested information.
            :param subject: The patient for which the request was processed.
            :param encounter: The encounter during which this response was created or to which the creation
        of this record is tightly associated.
            :param occurrenceDateTime: Indicates when the guidance response was processed.
            :param performer: Provides a reference to the device that performed the guidance.
            :param reasonCode: Describes the reason for the guidance response in coded or textual form.
            :param reasonReference: Indicates the reason the request was initiated. This is typically provided as
        a parameter to the evaluation and echoed by the service, although for some use
        cases, such as subscription- or event-based scenarios, it may provide an
        indication of the cause for the response.
            :param note: Provides a mechanism to communicate additional information about the response.
            :param evaluationMessage: Messages resulting from the evaluation of the artifact or artifacts. As part
        of evaluating the request, the engine may produce informational or warning
        messages. These messages will be provided by this element.
            :param outputParameters: The output parameters of the evaluation, if any. Many modules will result in
        the return of specific resources such as procedure or communication requests
        that are returned as part of the operation result. However, modules may define
        specific outputs that would be returned as the result of the evaluation, and
        these would be returned in this element.
            :param result: The actions, if any, produced by the evaluation of the artifact.
            :param dataRequirement: If the evaluation could not be completed due to lack of information, or
        additional information would potentially result in a more accurate response,
        this element will a description of the data required in order to proceed with
        the evaluation. A subsequent request to the service should include this data.
            :param moduleUri: None
            :param moduleCanonical: None
            :param moduleCodeableConcept: None
        """
        super().__init__(
            resourceType="GuidanceResponse",
            id_=id_,
            meta=meta,
            extension=extension,
            requestIdentifier=requestIdentifier,
            identifier=identifier,
            status=status,
            subject=subject,
            encounter=encounter,
            occurrenceDateTime=occurrenceDateTime,
            performer=performer,
            reasonCode=reasonCode,
            reasonReference=reasonReference,
            note=note,
            evaluationMessage=evaluationMessage,
            outputParameters=outputParameters,
            result=result,
            dataRequirement=dataRequirement,
            moduleUri=moduleUri,
            moduleCanonical=moduleCanonical,
            moduleCodeableConcept=moduleCodeableConcept,
        )

    def get_schema(
        self, include_extension: bool
    ) -> Optional[Union[StructType, DataType]]:
        return GuidanceResponseSchema.get_schema(include_extension=include_extension)
