from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

# noinspection PyPackageRequirements
from pyspark.sql.types import StructType, DataType
from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.complex_types.meta import Meta
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId
from spark_auto_mapper_fhir.fhir_types.uri import FhirUri

from spark_auto_mapper_fhir.base_types.fhir_resource_base import FhirResourceBase
from spark_fhir_schemas.r4.resources.medicationadministration import (
    MedicationAdministrationSchema,
)

if TYPE_CHECKING:
    pass
    # identifier (Identifier)
    from spark_auto_mapper_fhir.complex_types.identifier import Identifier

    # instantiates (uri)
    # partOf (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for partOf
    from spark_auto_mapper_fhir.resources.procedure import Procedure

    # status (MedicationAdministration Status Codes)
    from spark_auto_mapper_fhir.value_sets.medication_administration_status_codes import (
        MedicationAdministrationStatusCodesCode,
    )

    # statusReason (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # Import for CodeableConcept for statusReason
    from spark_auto_mapper_fhir.value_sets.snomedct_reason_medication_not_given_codes import (
        SNOMEDCTReasonMedicationNotGivenCodesCode,
    )

    # End Import for CodeableConcept for statusReason
    # category (CodeableConcept)
    # Import for CodeableConcept for category
    from spark_auto_mapper_fhir.value_sets.medication_administration_category_codes import (
        MedicationAdministrationCategoryCodesCode,
    )

    # End Import for CodeableConcept for category
    # subject (Reference)
    # Imports for References for subject
    from spark_auto_mapper_fhir.resources.patient import Patient
    from spark_auto_mapper_fhir.resources.group import Group

    # context (Reference)
    # Imports for References for context
    from spark_auto_mapper_fhir.resources.encounter import Encounter
    from spark_auto_mapper_fhir.resources.episode_of_care import EpisodeOfCare

    # supportingInformation (Reference)
    # Imports for References for supportingInformation
    from spark_auto_mapper_fhir.resources.resource import Resource

    # performer (MedicationAdministration.Performer)
    from spark_auto_mapper_fhir.backbone_elements.medication_administration_performer import (
        MedicationAdministrationPerformer,
    )

    # reasonCode (CodeableConcept)
    # Import for CodeableConcept for reasonCode
    from spark_auto_mapper_fhir.value_sets.reason_medication_given_codes import (
        ReasonMedicationGivenCodesCode,
    )

    # End Import for CodeableConcept for reasonCode
    # reasonReference (Reference)
    # Imports for References for reasonReference
    from spark_auto_mapper_fhir.resources.condition import Condition
    from spark_auto_mapper_fhir.resources.observation import Observation
    from spark_auto_mapper_fhir.resources.diagnostic_report import DiagnosticReport

    # request (Reference)
    # Imports for References for request
    from spark_auto_mapper_fhir.resources.medication_request import MedicationRequest

    # device (Reference)
    # Imports for References for device
    from spark_auto_mapper_fhir.resources.device import Device

    # note (Annotation)
    from spark_auto_mapper_fhir.complex_types.annotation import Annotation

    # dosage (MedicationAdministration.Dosage)
    from spark_auto_mapper_fhir.backbone_elements.medication_administration_dosage import (
        MedicationAdministrationDosage,
    )

    # eventHistory (Reference)
    # Imports for References for eventHistory
    from spark_auto_mapper_fhir.resources.provenance import Provenance

    # medicationCodeableConcept (CodeableConcept)
    # Import for CodeableConcept for medicationCodeableConcept
    from spark_auto_mapper_fhir.value_sets.generic_type import GenericTypeCode

    # End Import for CodeableConcept for medicationCodeableConcept
    # medicationReference (Reference)
    # Imports for References for medicationReference
    from spark_auto_mapper_fhir.resources.medication import Medication

    # effectiveDateTime (dateTime)
    # effectivePeriod (Period)
    from spark_auto_mapper_fhir.complex_types.period import Period


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class MedicationAdministration(FhirResourceBase):
    """
    MedicationAdministration
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: FhirId,
        meta: Optional[Meta] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        identifier: Optional[FhirList[Identifier]] = None,
        instantiates: Optional[FhirList[FhirUri]] = None,
        partOf: Optional[
            FhirList[Reference[Union[MedicationAdministration, Procedure]]]
        ] = None,
        status: MedicationAdministrationStatusCodesCode,
        statusReason: Optional[
            FhirList[CodeableConcept[SNOMEDCTReasonMedicationNotGivenCodesCode]]
        ] = None,
        category: Optional[
            CodeableConcept[MedicationAdministrationCategoryCodesCode]
        ] = None,
        subject: Reference[Union[Patient, Group]],
        context: Optional[Reference[Union[Encounter, EpisodeOfCare]]] = None,
        supportingInformation: Optional[FhirList[Reference[Union[Resource]]]] = None,
        performer: Optional[FhirList[MedicationAdministrationPerformer]] = None,
        reasonCode: Optional[
            FhirList[CodeableConcept[ReasonMedicationGivenCodesCode]]
        ] = None,
        reasonReference: Optional[
            FhirList[Reference[Union[Condition, Observation, DiagnosticReport]]]
        ] = None,
        request: Optional[Reference[Union[MedicationRequest]]] = None,
        device: Optional[FhirList[Reference[Union[Device]]]] = None,
        note: Optional[FhirList[Annotation]] = None,
        dosage: Optional[MedicationAdministrationDosage] = None,
        eventHistory: Optional[FhirList[Reference[Union[Provenance]]]] = None,
        medicationCodeableConcept: Optional[CodeableConcept[GenericTypeCode]] = None,
        medicationReference: Optional[Reference[Union[Medication]]] = None,
        effectiveDateTime: Optional[FhirDateTime] = None,
        effectivePeriod: Optional[Period] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param meta: Meta
            :param extension: extensions
            :param identifier: Identifiers associated with this Medication Administration that are defined by
        business processes and/or used to refer to it when a direct URL reference to
        the resource itself is not appropriate. They are business identifiers assigned
        to this resource by the performer or other systems and remain constant as the
        resource is updated and propagates from server to server.
            :param instantiates: A protocol, guideline, orderset, or other definition that was adhered to in
        whole or in part by this event.
            :param partOf: A larger event of which this particular event is a component or step.
            :param status: Will generally be set to show that the administration has been completed.  For
        some long running administrations such as infusions, it is possible for an
        administration to be started but not completed or it may be paused while some
        other process is under way.
            :param statusReason: A code indicating why the administration was not performed.
            :param category: Indicates where the medication is expected to be consumed or administered.
            :param subject: The person or animal or group receiving the medication.
            :param context: The visit, admission, or other contact between patient and health care
        provider during which the medication administration was performed.
            :param supportingInformation: Additional information (for example, patient height and weight) that supports
        the administration of the medication.
            :param performer: Indicates who or what performed the medication administration and how they
        were involved.
            :param reasonCode: A code indicating why the medication was given.
            :param reasonReference: Condition or observation that supports why the medication was administered.
            :param request: The original request, instruction or authority to perform the administration.
            :param device: The device used in administering the medication to the patient.  For example,
        a particular infusion pump.
            :param note: Extra information about the medication administration that is not conveyed by
        the other attributes.
            :param dosage: Describes the medication dosage information details e.g. dose, rate, site,
        route, etc.
            :param eventHistory: A summary of the events of interest that have occurred, such as when the
        administration was verified.
            :param medicationCodeableConcept: None
            :param medicationReference: None
            :param effectiveDateTime: None
            :param effectivePeriod: None
        """
        super().__init__(
            resourceType="MedicationAdministration",
            id_=id_,
            meta=meta,
            extension=extension,
            identifier=identifier,
            instantiates=instantiates,
            partOf=partOf,
            status=status,
            statusReason=statusReason,
            category=category,
            subject=subject,
            context=context,
            supportingInformation=supportingInformation,
            performer=performer,
            reasonCode=reasonCode,
            reasonReference=reasonReference,
            request=request,
            device=device,
            note=note,
            dosage=dosage,
            eventHistory=eventHistory,
            medicationCodeableConcept=medicationCodeableConcept,
            medicationReference=medicationReference,
            effectiveDateTime=effectiveDateTime,
            effectivePeriod=effectivePeriod,
        )

    def get_schema(
        self, include_extension: bool
    ) -> Optional[Union[StructType, DataType]]:
        return MedicationAdministrationSchema.get_schema(
            include_extension=include_extension
        )
