from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

# noinspection PyPackageRequirements
from pyspark.sql.types import StructType, DataType
from spark_auto_mapper_fhir.fhir_types.boolean import FhirBoolean
from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.complex_types.meta import Meta
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId
from spark_auto_mapper_fhir.fhir_types.uri import FhirUri

from spark_auto_mapper_fhir.base_types.fhir_resource_base import FhirResourceBase
from spark_fhir_schemas.r4.resources.operationdefinition import (
    OperationDefinitionSchema,
)

if TYPE_CHECKING:
    pass
    # url (uri)
    # version (string)
    # name (string)
    # title (string)
    # status (PublicationStatus)
    from spark_auto_mapper_fhir.value_sets.publication_status import (
        PublicationStatusCode,
    )

    # kind (OperationKind)
    from spark_auto_mapper_fhir.value_sets.operation_kind import OperationKindCode

    # experimental (boolean)
    # date (dateTime)
    # publisher (string)
    # contact (ContactDetail)
    from spark_auto_mapper_fhir.complex_types.contact_detail import ContactDetail

    # description (markdown)
    from spark_auto_mapper_fhir.fhir_types.markdown import FhirMarkdown

    # useContext (UsageContext)
    from spark_auto_mapper_fhir.complex_types.usage_context import UsageContext

    # jurisdiction (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # Import for CodeableConcept for jurisdiction
    from spark_auto_mapper_fhir.value_sets.jurisdiction_value_set import (
        JurisdictionValueSetCode,
    )

    # End Import for CodeableConcept for jurisdiction
    # purpose (markdown)
    # affectsState (boolean)
    # code (code)
    from spark_auto_mapper_fhir.complex_types.code import code

    # comment (markdown)
    # base (canonical)
    from spark_auto_mapper_fhir.fhir_types.canonical import FhirCanonical

    # resource (ResourceType)
    from spark_auto_mapper_fhir.value_sets.resource_type import ResourceTypeCode

    # system (boolean)
    # type_ (boolean)
    # instance (boolean)
    # inputProfile (canonical)
    # outputProfile (canonical)
    # parameter (OperationDefinition.Parameter)
    from spark_auto_mapper_fhir.backbone_elements.operation_definition_parameter import (
        OperationDefinitionParameter,
    )

    # overload (OperationDefinition.Overload)
    from spark_auto_mapper_fhir.backbone_elements.operation_definition_overload import (
        OperationDefinitionOverload,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class OperationDefinition(FhirResourceBase):
    """
    OperationDefinition
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: FhirId,
        meta: Optional[Meta] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        url: Optional[FhirUri] = None,
        version: Optional[FhirString] = None,
        name: FhirString,
        title: Optional[FhirString] = None,
        status: PublicationStatusCode,
        kind: OperationKindCode,
        experimental: Optional[FhirBoolean] = None,
        date: Optional[FhirDateTime] = None,
        publisher: Optional[FhirString] = None,
        contact: Optional[FhirList[ContactDetail]] = None,
        description: Optional[FhirMarkdown] = None,
        useContext: Optional[FhirList[UsageContext]] = None,
        jurisdiction: Optional[
            FhirList[CodeableConcept[JurisdictionValueSetCode]]
        ] = None,
        purpose: Optional[FhirMarkdown] = None,
        affectsState: Optional[FhirBoolean] = None,
        code: code,
        comment: Optional[FhirMarkdown] = None,
        base: Optional[FhirCanonical] = None,
        resource: Optional[FhirList[ResourceTypeCode]] = None,
        system: FhirBoolean,
        type_: FhirBoolean,
        instance: FhirBoolean,
        inputProfile: Optional[FhirCanonical] = None,
        outputProfile: Optional[FhirCanonical] = None,
        parameter: Optional[FhirList[OperationDefinitionParameter]] = None,
        overload: Optional[FhirList[OperationDefinitionOverload]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param meta: Meta
            :param extension: extensions
            :param url: An absolute URI that is used to identify this operation definition when it is
        referenced in a specification, model, design or an instance; also called its
        canonical identifier. This SHOULD be globally unique and SHOULD be a literal
        address at which at which an authoritative instance of this operation
        definition is (or will be) published. This URL can be the target of a
        canonical reference. It SHALL remain the same when the operation definition is
        stored on different servers.
            :param version: The identifier that is used to identify this version of the operation
        definition when it is referenced in a specification, model, design or
        instance. This is an arbitrary value managed by the operation definition
        author and is not expected to be globally unique. For example, it might be a
        timestamp (e.g. yyyymmdd) if a managed version is not available. There is also
        no expectation that versions can be placed in a lexicographical sequence.
            :param name: A natural language name identifying the operation definition. This name should
        be usable as an identifier for the module by machine processing applications
        such as code generation.
            :param title: A short, descriptive, user-friendly title for the operation definition.
            :param status: The status of this operation definition. Enables tracking the life-cycle of
        the content.
            :param kind: Whether this is an operation or a named query.
            :param experimental: A Boolean value to indicate that this operation definition is authored for
        testing purposes (or education/evaluation/marketing) and is not intended to be
        used for genuine usage.
            :param date: The date  (and optionally time) when the operation definition was published.
        The date must change when the business version changes and it must change if
        the status code changes. In addition, it should change when the substantive
        content of the operation definition changes.
            :param publisher: The name of the organization or individual that published the operation
        definition.
            :param contact: Contact details to assist a user in finding and communicating with the
        publisher.
            :param description: A free text natural language description of the operation definition from a
        consumer's perspective.
            :param useContext: The content was developed with a focus and intent of supporting the contexts
        that are listed. These contexts may be general categories (gender, age, ...)
        or may be references to specific programs (insurance plans, studies, ...) and
        may be used to assist with indexing and searching for appropriate operation
        definition instances.
            :param jurisdiction: A legal or geographic region in which the operation definition is intended to
        be used.
            :param purpose: Explanation of why this operation definition is needed and why it has been
        designed as it has.
            :param affectsState: Whether the operation affects state. Side effects such as producing audit
        trail entries do not count as 'affecting  state'.
            :param code: The name used to invoke the operation.
            :param comment: Additional information about how to use this operation or named query.
            :param base: Indicates that this operation definition is a constraining profile on the
        base.
            :param resource: The types on which this operation can be executed.
            :param system: Indicates whether this operation or named query can be invoked at the system
        level (e.g. without needing to choose a resource type for the context).
            :param type_: Indicates whether this operation or named query can be invoked at the resource
        type level for any given resource type level (e.g. without needing to choose a
        specific resource id for the context).
            :param instance: Indicates whether this operation can be invoked on a particular instance of
        one of the given types.
            :param inputProfile: Additional validation information for the in parameters - a single profile
        that covers all the parameters. The profile is a constraint on the parameters
        resource as a whole.
            :param outputProfile: Additional validation information for the out parameters - a single profile
        that covers all the parameters. The profile is a constraint on the parameters
        resource.
            :param parameter: The parameters for the operation/query.
            :param overload: Defines an appropriate combination of parameters to use when invoking this
        operation, to help code generators when generating overloaded parameter sets
        for this operation.
        """
        super().__init__(
            resourceType="OperationDefinition",
            id_=id_,
            meta=meta,
            extension=extension,
            url=url,
            version=version,
            name=name,
            title=title,
            status=status,
            kind=kind,
            experimental=experimental,
            date=date,
            publisher=publisher,
            contact=contact,
            description=description,
            useContext=useContext,
            jurisdiction=jurisdiction,
            purpose=purpose,
            affectsState=affectsState,
            code=code,
            comment=comment,
            base=base,
            resource=resource,
            system=system,
            type_=type_,
            instance=instance,
            inputProfile=inputProfile,
            outputProfile=outputProfile,
            parameter=parameter,
            overload=overload,
        )

    def get_schema(
        self, include_extension: bool
    ) -> Optional[Union[StructType, DataType]]:
        return OperationDefinitionSchema.get_schema(include_extension=include_extension)
