from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

# noinspection PyPackageRequirements
from pyspark.sql.types import StructType, DataType
from spark_auto_mapper_fhir.fhir_types.boolean import FhirBoolean
from spark_auto_mapper_fhir.fhir_types.date import FhirDate
from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.integer import FhirInteger
from spark_auto_mapper_fhir.complex_types.meta import Meta
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_resource_base import FhirResourceBase
from spark_fhir_schemas.r4.resources.patient import PatientSchema

if TYPE_CHECKING:
    pass
    # identifier (Identifier)
    from spark_auto_mapper_fhir.complex_types.identifier import Identifier

    # active (boolean)
    # name (HumanName)
    from spark_auto_mapper_fhir.complex_types.human_name import HumanName

    # telecom (ContactPoint)
    from spark_auto_mapper_fhir.complex_types.contact_point import ContactPoint

    # gender (AdministrativeGender)
    from spark_auto_mapper_fhir.value_sets.administrative_gender import (
        AdministrativeGenderCode,
    )

    # birthDate (date)
    # address (Address)
    from spark_auto_mapper_fhir.complex_types.address import Address

    # maritalStatus (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # Import for CodeableConcept for maritalStatus
    from spark_auto_mapper_fhir.value_sets.marital_status_codes import (
        MaritalStatusCodesCode,
    )

    # End Import for CodeableConcept for maritalStatus
    # photo (Attachment)
    from spark_auto_mapper_fhir.complex_types.attachment import Attachment

    # contact (Patient.Contact)
    from spark_auto_mapper_fhir.backbone_elements.patient_contact import PatientContact

    # communication (Patient.Communication)
    from spark_auto_mapper_fhir.backbone_elements.patient_communication import (
        PatientCommunication,
    )

    # generalPractitioner (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for generalPractitioner
    from spark_auto_mapper_fhir.resources.organization import Organization
    from spark_auto_mapper_fhir.resources.practitioner import Practitioner
    from spark_auto_mapper_fhir.resources.practitioner_role import PractitionerRole

    # managingOrganization (Reference)
    # Imports for References for managingOrganization
    # link (Patient.Link)
    from spark_auto_mapper_fhir.backbone_elements.patient_link import PatientLink

    # deceasedBoolean (boolean)
    # deceasedDateTime (dateTime)
    # multipleBirthBoolean (boolean)
    # multipleBirthInteger (integer)


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class Patient(FhirResourceBase):
    """
    Patient
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: FhirId,
        meta: Optional[Meta] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        identifier: Optional[FhirList[Identifier]] = None,
        active: Optional[FhirBoolean] = None,
        name: Optional[FhirList[HumanName]] = None,
        telecom: Optional[FhirList[ContactPoint]] = None,
        gender: Optional[AdministrativeGenderCode] = None,
        birthDate: Optional[FhirDate] = None,
        address: Optional[FhirList[Address]] = None,
        maritalStatus: Optional[CodeableConcept[MaritalStatusCodesCode]] = None,
        photo: Optional[FhirList[Attachment]] = None,
        contact: Optional[FhirList[PatientContact]] = None,
        communication: Optional[FhirList[PatientCommunication]] = None,
        generalPractitioner: Optional[
            FhirList[Reference[Union[Organization, Practitioner, PractitionerRole]]]
        ] = None,
        managingOrganization: Optional[Reference[Union[Organization]]] = None,
        link: Optional[FhirList[PatientLink]] = None,
        deceasedBoolean: Optional[FhirBoolean] = None,
        deceasedDateTime: Optional[FhirDateTime] = None,
        multipleBirthBoolean: Optional[FhirBoolean] = None,
        multipleBirthInteger: Optional[FhirInteger] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param meta: Meta
            :param extension: extensions
            :param identifier: An identifier for this patient.
            :param active: Whether this patient record is in active use.
        Many systems use this property to mark as non-current patients, such as those
        that have not been seen for a period of time based on an organization's
        business rules.

        It is often used to filter patient lists to exclude inactive patients

        Deceased patients may also be marked as inactive for the same reasons, but may
        be active for some time after death.
            :param name: A name associated with the individual.
            :param telecom: A contact detail (e.g. a telephone number or an email address) by which the
        individual may be contacted.
            :param gender: Administrative Gender - the gender that the patient is considered to have for
        administration and record keeping purposes.
            :param birthDate: The date of birth for the individual.
            :param address: An address for the individual.
            :param maritalStatus: This field contains a patient's most recent marital (civil) status.
            :param photo: Image of the patient.
            :param contact: A contact party (e.g. guardian, partner, friend) for the patient.
            :param communication: A language which may be used to communicate with the patient about his or her
        health.
            :param generalPractitioner: Patient's nominated care provider.
            :param managingOrganization: Organization that is the custodian of the patient record.
            :param link: Link to another patient resource that concerns the same actual patient.
            :param deceasedBoolean: None
            :param deceasedDateTime: None
            :param multipleBirthBoolean: None
            :param multipleBirthInteger: None
        """
        super().__init__(
            resourceType="Patient",
            id_=id_,
            meta=meta,
            extension=extension,
            identifier=identifier,
            active=active,
            name=name,
            telecom=telecom,
            gender=gender,
            birthDate=birthDate,
            address=address,
            maritalStatus=maritalStatus,
            photo=photo,
            contact=contact,
            communication=communication,
            generalPractitioner=generalPractitioner,
            managingOrganization=managingOrganization,
            link=link,
            deceasedBoolean=deceasedBoolean,
            deceasedDateTime=deceasedDateTime,
            multipleBirthBoolean=multipleBirthBoolean,
            multipleBirthInteger=multipleBirthInteger,
        )

    def get_schema(
        self, include_extension: bool
    ) -> Optional[Union[StructType, DataType]]:
        return PatientSchema.get_schema(include_extension=include_extension)
