from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

# noinspection PyPackageRequirements
from pyspark.sql.types import StructType, DataType
from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.complex_types.meta import Meta
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_resource_base import FhirResourceBase
from spark_fhir_schemas.r4.resources.questionnaireresponse import (
    QuestionnaireResponseSchema,
)

if TYPE_CHECKING:
    pass
    # identifier (Identifier)
    from spark_auto_mapper_fhir.complex_types.identifier import Identifier

    # basedOn (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for basedOn
    from spark_auto_mapper_fhir.resources.care_plan import CarePlan
    from spark_auto_mapper_fhir.resources.service_request import ServiceRequest

    # partOf (Reference)
    # Imports for References for partOf
    from spark_auto_mapper_fhir.resources.observation import Observation
    from spark_auto_mapper_fhir.resources.procedure import Procedure

    # questionnaire (canonical)
    from spark_auto_mapper_fhir.fhir_types.canonical import FhirCanonical

    # status (QuestionnaireResponseStatus)
    from spark_auto_mapper_fhir.value_sets.questionnaire_response_status import (
        QuestionnaireResponseStatusCode,
    )

    # subject (Reference)
    # Imports for References for subject
    from spark_auto_mapper_fhir.resources.resource import Resource

    # encounter (Reference)
    # Imports for References for encounter
    from spark_auto_mapper_fhir.resources.encounter import Encounter

    # authored (dateTime)
    # author (Reference)
    # Imports for References for author
    from spark_auto_mapper_fhir.resources.device import Device
    from spark_auto_mapper_fhir.resources.practitioner import Practitioner
    from spark_auto_mapper_fhir.resources.practitioner_role import PractitionerRole
    from spark_auto_mapper_fhir.resources.patient import Patient
    from spark_auto_mapper_fhir.resources.related_person import RelatedPerson
    from spark_auto_mapper_fhir.resources.organization import Organization

    # source (Reference)
    # Imports for References for source
    # item (QuestionnaireResponse.Item)
    from spark_auto_mapper_fhir.backbone_elements.questionnaire_response_item import (
        QuestionnaireResponseItem,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class QuestionnaireResponse(FhirResourceBase):
    """
    QuestionnaireResponse
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: FhirId,
        meta: Optional[Meta] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        identifier: Optional[Identifier] = None,
        basedOn: Optional[FhirList[Reference[Union[CarePlan, ServiceRequest]]]] = None,
        partOf: Optional[FhirList[Reference[Union[Observation, Procedure]]]] = None,
        questionnaire: Optional[FhirCanonical] = None,
        status: QuestionnaireResponseStatusCode,
        subject: Optional[Reference[Union[Resource]]] = None,
        encounter: Optional[Reference[Union[Encounter]]] = None,
        authored: Optional[FhirDateTime] = None,
        author: Optional[
            Reference[
                Union[
                    Device,
                    Practitioner,
                    PractitionerRole,
                    Patient,
                    RelatedPerson,
                    Organization,
                ]
            ]
        ] = None,
        source: Optional[
            Reference[Union[Patient, Practitioner, PractitionerRole, RelatedPerson]]
        ] = None,
        item: Optional[FhirList[QuestionnaireResponseItem]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param meta: Meta
            :param extension: extensions
            :param identifier: A business identifier assigned to a particular completed (or partially
        completed) questionnaire.
            :param basedOn: The order, proposal or plan that is fulfilled in whole or in part by this
        QuestionnaireResponse.  For example, a ServiceRequest seeking an intake
        assessment or a decision support recommendation to assess for post-partum
        depression.
            :param partOf: A procedure or observation that this questionnaire was performed as part of
        the execution of.  For example, the surgery a checklist was executed as part
        of.
            :param questionnaire: The Questionnaire that defines and organizes the questions for which answers
        are being provided.
            :param status: The position of the questionnaire response within its overall lifecycle.
            :param subject: The subject of the questionnaire response.  This could be a patient,
        organization, practitioner, device, etc.  This is who/what the answers apply
        to, but is not necessarily the source of information.
            :param encounter: The Encounter during which this questionnaire response was created or to which
        the creation of this record is tightly associated.
            :param authored: The date and/or time that this set of answers were last changed.
            :param author: Person who received the answers to the questions in the QuestionnaireResponse
        and recorded them in the system.
            :param source: The person who answered the questions about the subject.
            :param item: A group or question item from the original questionnaire for which answers are
        provided.
        """
        super().__init__(
            resourceType="QuestionnaireResponse",
            id_=id_,
            meta=meta,
            extension=extension,
            identifier=identifier,
            basedOn=basedOn,
            partOf=partOf,
            questionnaire=questionnaire,
            status=status,
            subject=subject,
            encounter=encounter,
            authored=authored,
            author=author,
            source=source,
            item=item,
        )

    def get_schema(
        self, include_extension: bool
    ) -> Optional[Union[StructType, DataType]]:
        return QuestionnaireResponseSchema.get_schema(
            include_extension=include_extension
        )
