from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

# noinspection PyPackageRequirements
from pyspark.sql.types import StructType, DataType
from spark_auto_mapper_fhir.fhir_types.boolean import FhirBoolean
from spark_auto_mapper_fhir.fhir_types.date import FhirDate
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.complex_types.meta import Meta
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_resource_base import FhirResourceBase
from spark_fhir_schemas.r4.resources.relatedperson import RelatedPersonSchema

if TYPE_CHECKING:
    pass
    # identifier (Identifier)
    from spark_auto_mapper_fhir.complex_types.identifier import Identifier

    # active (boolean)
    # patient (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for patient
    from spark_auto_mapper_fhir.resources.patient import Patient

    # relationship (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # Import for CodeableConcept for relationship
    from spark_auto_mapper_fhir.value_sets.patient_relationship_type import (
        PatientRelationshipTypeCode,
    )

    # End Import for CodeableConcept for relationship
    # name (HumanName)
    from spark_auto_mapper_fhir.complex_types.human_name import HumanName

    # telecom (ContactPoint)
    from spark_auto_mapper_fhir.complex_types.contact_point import ContactPoint

    # gender (AdministrativeGender)
    from spark_auto_mapper_fhir.value_sets.administrative_gender import (
        AdministrativeGenderCode,
    )

    # birthDate (date)
    # address (Address)
    from spark_auto_mapper_fhir.complex_types.address import Address

    # photo (Attachment)
    from spark_auto_mapper_fhir.complex_types.attachment import Attachment

    # period (Period)
    from spark_auto_mapper_fhir.complex_types.period import Period

    # communication (RelatedPerson.Communication)
    from spark_auto_mapper_fhir.backbone_elements.related_person_communication import (
        RelatedPersonCommunication,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class RelatedPerson(FhirResourceBase):
    """
    RelatedPerson
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: FhirId,
        meta: Optional[Meta] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        identifier: Optional[FhirList[Identifier]] = None,
        active: Optional[FhirBoolean] = None,
        patient: Reference[Union[Patient]],
        relationship: Optional[
            FhirList[CodeableConcept[PatientRelationshipTypeCode]]
        ] = None,
        name: Optional[FhirList[HumanName]] = None,
        telecom: Optional[FhirList[ContactPoint]] = None,
        gender: Optional[AdministrativeGenderCode] = None,
        birthDate: Optional[FhirDate] = None,
        address: Optional[FhirList[Address]] = None,
        photo: Optional[FhirList[Attachment]] = None,
        period: Optional[Period] = None,
        communication: Optional[FhirList[RelatedPersonCommunication]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param meta: Meta
            :param extension: extensions
            :param identifier: Identifier for a person within a particular scope.
            :param active: Whether this related person record is in active use.
            :param patient: The patient this person is related to.
            :param relationship: The nature of the relationship between a patient and the related person.
            :param name: A name associated with the person.
            :param telecom: A contact detail for the person, e.g. a telephone number or an email address.
            :param gender: Administrative Gender - the gender that the person is considered to have for
        administration and record keeping purposes.
            :param birthDate: The date on which the related person was born.
            :param address: Address where the related person can be contacted or visited.
            :param photo: Image of the person.
            :param period: The period of time during which this relationship is or was active. If there
        are no dates defined, then the interval is unknown.
            :param communication: A language which may be used to communicate with about the patient's health.
        """
        super().__init__(
            resourceType="RelatedPerson",
            id_=id_,
            meta=meta,
            extension=extension,
            identifier=identifier,
            active=active,
            patient=patient,
            relationship=relationship,
            name=name,
            telecom=telecom,
            gender=gender,
            birthDate=birthDate,
            address=address,
            photo=photo,
            period=period,
            communication=communication,
        )

    def get_schema(
        self, include_extension: bool
    ) -> Optional[Union[StructType, DataType]]:
        return RelatedPersonSchema.get_schema(include_extension=include_extension)
