from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

# noinspection PyPackageRequirements
from pyspark.sql.types import StructType, DataType
from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.complex_types.meta import Meta
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_resource_base import FhirResourceBase
from spark_fhir_schemas.r4.resources.specimen import SpecimenSchema

if TYPE_CHECKING:
    pass
    # identifier (Identifier)
    from spark_auto_mapper_fhir.complex_types.identifier import Identifier

    # accessionIdentifier (Identifier)
    # status (SpecimenStatus)
    from spark_auto_mapper_fhir.value_sets.specimen_status import SpecimenStatusCode

    # type_ (CodeableConcept)
    from spark_auto_mapper_fhir.complex_types.codeable_concept import CodeableConcept

    # Import for CodeableConcept for type_
    from spark_auto_mapper_fhir.value_sets.v2_0487 import V2_0487

    # End Import for CodeableConcept for type_
    # subject (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for subject
    from spark_auto_mapper_fhir.resources.patient import Patient
    from spark_auto_mapper_fhir.resources.group import Group
    from spark_auto_mapper_fhir.resources.device import Device
    from spark_auto_mapper_fhir.resources.substance import Substance
    from spark_auto_mapper_fhir.resources.location import Location

    # receivedTime (dateTime)
    # parent (Reference)
    # Imports for References for parent
    # request (Reference)
    # Imports for References for request
    from spark_auto_mapper_fhir.resources.service_request import ServiceRequest

    # collection (Specimen.Collection)
    from spark_auto_mapper_fhir.backbone_elements.specimen_collection import (
        SpecimenCollection,
    )

    # processing (Specimen.Processing)
    from spark_auto_mapper_fhir.backbone_elements.specimen_processing import (
        SpecimenProcessing,
    )

    # container (Specimen.Container)
    from spark_auto_mapper_fhir.backbone_elements.specimen_container import (
        SpecimenContainer,
    )

    # condition (CodeableConcept)
    # Import for CodeableConcept for condition
    from spark_auto_mapper_fhir.value_sets.v2_0493 import V2_0493

    # End Import for CodeableConcept for condition
    # note (Annotation)
    from spark_auto_mapper_fhir.complex_types.annotation import Annotation


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class Specimen(FhirResourceBase):
    """
    Specimen
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: FhirId,
        meta: Optional[Meta] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        identifier: Optional[FhirList[Identifier]] = None,
        accessionIdentifier: Optional[Identifier] = None,
        status: Optional[SpecimenStatusCode] = None,
        type_: Optional[CodeableConcept[V2_0487]] = None,
        subject: Optional[
            Reference[Union[Patient, Group, Device, Substance, Location]]
        ] = None,
        receivedTime: Optional[FhirDateTime] = None,
        parent: Optional[FhirList[Reference[Union[Specimen]]]] = None,
        request: Optional[FhirList[Reference[Union[ServiceRequest]]]] = None,
        collection: Optional[SpecimenCollection] = None,
        processing: Optional[FhirList[SpecimenProcessing]] = None,
        container: Optional[FhirList[SpecimenContainer]] = None,
        condition: Optional[FhirList[CodeableConcept[V2_0493]]] = None,
        note: Optional[FhirList[Annotation]] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param meta: Meta
            :param extension: extensions
            :param identifier: Id for specimen.
            :param accessionIdentifier: The identifier assigned by the lab when accessioning specimen(s). This is not
        necessarily the same as the specimen identifier, depending on local lab
        procedures.
            :param status: The availability of the specimen.
            :param type_: The kind of material that forms the specimen.
            :param subject: Where the specimen came from. This may be from patient(s), from a location
        (e.g., the source of an environmental sample), or a sampling of a substance or
        a device.
            :param receivedTime: Time when specimen was received for processing or testing.
            :param parent: Reference to the parent (source) specimen which is used when the specimen was
        either derived from or a component of another specimen.
            :param request: Details concerning a service request that required a specimen to be collected.
            :param collection: Details concerning the specimen collection.
            :param processing: Details concerning processing and processing steps for the specimen.
            :param container: The container holding the specimen.  The recursive nature of containers; i.e.
        blood in tube in tray in rack is not addressed here.
            :param condition: A mode or state of being that describes the nature of the specimen.
            :param note: To communicate any details or issues about the specimen or during the specimen
        collection. (for example: broken vial, sent with patient, frozen).
        """
        super().__init__(
            resourceType="Specimen",
            id_=id_,
            meta=meta,
            extension=extension,
            identifier=identifier,
            accessionIdentifier=accessionIdentifier,
            status=status,
            type_=type_,
            subject=subject,
            receivedTime=receivedTime,
            parent=parent,
            request=request,
            collection=collection,
            processing=processing,
            container=container,
            condition=condition,
            note=note,
        )

    def get_schema(
        self, include_extension: bool
    ) -> Optional[Union[StructType, DataType]]:
        return SpecimenSchema.get_schema(include_extension=include_extension)
