from __future__ import annotations
from typing import Optional, TYPE_CHECKING, Union

# noinspection PyPackageRequirements
from pyspark.sql.types import StructType, DataType
from spark_auto_mapper_fhir.fhir_types.date_time import FhirDateTime
from spark_auto_mapper_fhir.fhir_types.list import FhirList
from spark_auto_mapper_fhir.fhir_types.string import FhirString
from spark_auto_mapper_fhir.complex_types.meta import Meta
from spark_auto_mapper_fhir.extensions.extension_base import ExtensionBase
from spark_auto_mapper_fhir.fhir_types.id import FhirId

from spark_auto_mapper_fhir.base_types.fhir_resource_base import FhirResourceBase
from spark_fhir_schemas.r4.resources.testreport import TestReportSchema

if TYPE_CHECKING:
    pass
    # identifier (Identifier)
    from spark_auto_mapper_fhir.complex_types.identifier import Identifier

    # name (string)
    # status (TestReportStatus)
    from spark_auto_mapper_fhir.value_sets.test_report_status import (
        TestReportStatusCode,
    )

    # testScript (Reference)
    from spark_auto_mapper_fhir.complex_types.reference import Reference

    # Imports for References for testScript
    from spark_auto_mapper_fhir.resources.test_script import TestScript

    # result (TestReportResult)
    from spark_auto_mapper_fhir.value_sets.test_report_result import (
        TestReportResultCode,
    )

    # score (decimal)
    from spark_auto_mapper_fhir.fhir_types.decimal import FhirDecimal

    # tester (string)
    # issued (dateTime)
    # participant (TestReport.Participant)
    from spark_auto_mapper_fhir.backbone_elements.test_report_participant import (
        TestReportParticipant,
    )

    # setup (TestReport.Setup)
    from spark_auto_mapper_fhir.backbone_elements.test_report_setup import (
        TestReportSetup,
    )

    # test (TestReport.Test)
    from spark_auto_mapper_fhir.backbone_elements.test_report_test import TestReportTest

    # teardown (TestReport.Teardown)
    from spark_auto_mapper_fhir.backbone_elements.test_report_teardown import (
        TestReportTeardown,
    )


# This file is auto-generated by generate_classes so do not edit manually
# noinspection PyPep8Naming
class TestReport(FhirResourceBase):
    """
    TestReport
    """

    # noinspection PyPep8Naming
    def __init__(
        self,
        *,
        id_: FhirId,
        meta: Optional[Meta] = None,
        extension: Optional[FhirList[ExtensionBase]] = None,
        identifier: Optional[Identifier] = None,
        name: Optional[FhirString] = None,
        status: TestReportStatusCode,
        testScript: Reference[Union[TestScript]],
        result: TestReportResultCode,
        score: Optional[FhirDecimal] = None,
        tester: Optional[FhirString] = None,
        issued: Optional[FhirDateTime] = None,
        participant: Optional[FhirList[TestReportParticipant]] = None,
        setup: Optional[TestReportSetup] = None,
        test: Optional[FhirList[TestReportTest]] = None,
        teardown: Optional[TestReportTeardown] = None,
    ) -> None:
        """

            :param id_: id of resource
            :param meta: Meta
            :param extension: extensions
            :param identifier: Identifier for the TestScript assigned for external purposes outside the
        context of FHIR.
            :param name: A free text natural language name identifying the executed TestScript.
            :param status: The current state of this test report.
            :param testScript: Ideally this is an absolute URL that is used to identify the version-specific
        TestScript that was executed, matching the `TestScript.url`.
            :param result: The overall result from the execution of the TestScript.
            :param score: The final score (percentage of tests passed) resulting from the execution of
        the TestScript.
            :param tester: Name of the tester producing this report (Organization or individual).
            :param issued: When the TestScript was executed and this TestReport was generated.
            :param participant: A participant in the test execution, either the execution engine, a client, or
        a server.
            :param setup: The results of the series of required setup operations before the tests were
        executed.
            :param test: A test executed from the test script.
            :param teardown: The results of the series of operations required to clean up after all the
        tests were executed (successfully or otherwise).
        """
        super().__init__(
            resourceType="TestReport",
            id_=id_,
            meta=meta,
            extension=extension,
            identifier=identifier,
            name=name,
            status=status,
            testScript=testScript,
            result=result,
            score=score,
            tester=tester,
            issued=issued,
            participant=participant,
            setup=setup,
            test=test,
            teardown=teardown,
        )

    def get_schema(
        self, include_extension: bool
    ) -> Optional[Union[StructType, DataType]]:
        return TestReportSchema.get_schema(include_extension=include_extension)
